@@ -40,7 +40,14 @@ var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
         MozXULElement.parseXULToFragment(
           `
           <hbox align="start" flex="1">
-            <image class="calendar-invitations-richlistitem-icon"/>
+            <!-- Note: The wrapper div is only here because the XUL box does not
+               - properly crop img elements with CSS object-fit and
+               - object-position. Should be removed when converting the parent
+               - element to HTML. -->
+            <html:div>
+              <html:img class="calendar-invitations-richlistitem-icon"
+                        src="chrome://calendar/skin/shared/calendar-invitations-dialog-list-images.png" />
+            </html:div>
             <vbox flex="1">
               <label class="calendar-invitations-richlistitem-title" crop="end"/>
               <label class="calendar-invitations-richlistitem-date" crop="end"/>
@@ -73,7 +80,6 @@ var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
 
     set calendarItem(val) {
       this.setCalendarItem(val);
-      return val;
     }
 
     get calendarItem() {
@@ -82,7 +88,6 @@ var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
 
     set initialParticipationStatus(val) {
       this.mInitialParticipationStatus = val;
-      return val;
     }
 
     get initialParticipationStatus() {
@@ -92,8 +97,12 @@ var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
     set participationStatus(val) {
       this.mParticipationStatus = val;
       let icon = this.querySelector(".calendar-invitations-richlistitem-icon");
+      // Status attribute changes the image region in CSS.
       icon.setAttribute("status", val);
-      return val;
+      document.l10n.setAttributes(
+        icon,
+        `calendar-invitation-current-participation-status-icon-${val.toLowerCase()}`
+      );
     }
 
     get participationStatus() {
@@ -172,9 +181,8 @@ var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
     }
 
     setCalendarItemParticipationStatus(item, status) {
-      let calendar = cal.wrapInstance(item.calendar, Ci.calISchedulingSupport);
-      if (calendar) {
-        let att = calendar.getInvitedAttendee(item);
+      if (item.calendar?.supportsScheduling) {
+        let att = item.calendar.getSchedulingSupport().getInvitedAttendee(item);
         if (att) {
           let att_ = att.clone();
           att_.participationStatus = status;
@@ -207,7 +215,7 @@ var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
  */
 function onLoad() {
   let operationListener = {
-    QueryInterface: ChromeUtils.generateQI([Ci.calIOperationListener]),
+    QueryInterface: ChromeUtils.generateQI(["calIOperationListener"]),
     onOperationComplete(aCalendar, aStatus, aOperationType, aId, aDetail) {
       let updatingBox = document.getElementById("updating-box");
       updatingBox.setAttribute("hidden", "true");
