@@ -2,13 +2,14 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* globals cal MozXULElement setBooleanAttribute */
+/* globals cal MozXULElement */
 
 "use strict";
 
 // Wrap in a block to prevent leaking to window scope.
 {
   const { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
+  const { XPCOMUtils } = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
 
   /**
    * MiniMonth Calendar: day-of-month grid component.
@@ -24,7 +25,6 @@
    * @implements {calIOperationListener}
    * @implements {calIObserver}
    * @implements {calICompositeObserver}
-   * @implements {nsIObserver}
    */
   class CalendarMinimonth extends MozXULElement {
     constructor() {
@@ -33,13 +33,31 @@
       this.calIObserver = this.getCustomInterfaceCallback(Ci.calIObserver);
       this.calICompositeObserver = this.getCustomInterfaceCallback(Ci.calICompositeObserver);
       this.calIOperationListener = this.getCustomInterfaceCallback(Ci.calIOperationListener);
-      this.nsIObserver = this.getCustomInterfaceCallback(Ci.nsIObserver);
+
+      let onPreferenceChanged = () => {
+        delete this.mDayMap; // Days have moved, force a refresh of the grid.
+        this.refreshDisplay();
+      };
+
+      XPCOMUtils.defineLazyPreferenceGetter(
+        this,
+        "weekStart",
+        "calendar.week.start",
+        0,
+        onPreferenceChanged
+      );
+      XPCOMUtils.defineLazyPreferenceGetter(
+        this,
+        "showWeekNumber",
+        "calendar.view-minimonth.showWeekNumber",
+        true,
+        onPreferenceChanged
+      );
     }
 
     static get inheritedAttributes() {
       return {
         ".minimonth-header": "readonly,month,year",
-        ".minimonth-month-name": "selectedIndex=month",
         ".minimonth-year-name": "value=year",
       };
     }
@@ -50,43 +68,24 @@
       }
 
       const minimonthHeader = `
-        <hbox class="minimonth-header minimonth-month-box">
+        <hbox class="minimonth-header minimonth-month-box" align="center">
           <toolbarbutton class="months-back-button minimonth-nav-btns"
                          dir="-1"
-                         oncommand="this.kMinimonth.advanceMonth(parseInt(this.getAttribute('dir'), 10))"
                          tooltiptext="&onemonthbackward.tooltip;"></toolbarbutton>
-          <deck class="monthheader minimonth-month-name"
-                tabindex="-1">
-            <html:div>&month.1.name;</html:div>
-            <html:div>&month.2.name;</html:div>
-            <html:div>&month.3.name;</html:div>
-            <html:div>&month.4.name;</html:div>
-            <html:div>&month.5.name;</html:div>
-            <html:div>&month.6.name;</html:div>
-            <html:div>&month.7.name;</html:div>
-            <html:div>&month.8.name;</html:div>
-            <html:div>&month.9.name;</html:div>
-            <html:div>&month.10.name;</html:div>
-            <html:div>&month.11.name;</html:div>
-            <html:div>&month.12.name;</html:div>
-          </deck>
+          <label class="minimonth-month-name" tabindex="-1"/>
           <toolbarbutton class="months-forward-button minimonth-nav-btns"
                          dir="1"
-                         oncommand="this.kMinimonth.advanceMonth(parseInt(this.getAttribute('dir'), 10))"
                          tooltiptext="&onemonthforward.tooltip;"></toolbarbutton>
           <toolbarbutton class="years-back-button minimonth-nav-btns"
                          dir="-1"
-                         oncommand="this.kMinimonth.advanceYear(parseInt(this.getAttribute('dir'), 10))"
                          tooltiptext="&oneyearbackward.tooltip;"></toolbarbutton>
           <label class="yearcell minimonth-year-name" tabindex="-1"></label>
           <toolbarbutton class="years-forward-button minimonth-nav-btns"
                          dir="1"
-                         oncommand="this.kMinimonth.advanceYear(parseInt(this.getAttribute('dir'), 10))"
                          tooltiptext="&oneyearforward.tooltip;"></toolbarbutton>
           <spacer flex="1"></spacer>
           <toolbarbutton class="today-button minimonth-nav-btns"
                          dir="0"
-                         oncommand="this.kMinimonth.value = new Date();"
                          tooltiptext="&showToday.tooltip;"></toolbarbutton>
         </hbox>
       `;
@@ -135,12 +134,21 @@
       this.setAttribute("orient", "vertical");
 
       // Set up header buttons.
-      const kMinimonth = this.closest("calendar-minimonth");
-      this.querySelector(".months-back-button").kMinimonth = kMinimonth;
-      this.querySelector(".months-forward-button").kMinimonth = kMinimonth;
-      this.querySelector(".years-back-button").kMinimonth = kMinimonth;
-      this.querySelector(".years-forward-button").kMinimonth = kMinimonth;
-      this.querySelector(".today-button").kMinimonth = kMinimonth;
+      this.querySelector(".months-back-button").addEventListener("command", () =>
+        this.advanceMonth(-1)
+      );
+      this.querySelector(".months-forward-button").addEventListener("command", () =>
+        this.advanceMonth(1)
+      );
+      this.querySelector(".years-back-button").addEventListener("command", () =>
+        this.advanceYear(-1)
+      );
+      this.querySelector(".years-forward-button").addEventListener("command", () =>
+        this.advanceYear(1)
+      );
+      this.querySelector(".today-button").addEventListener("command", () => {
+        this.value = new Date();
+      });
 
       this.mDayMap = null;
       this.mValue = null;
@@ -148,34 +156,35 @@
       this.mExtraDate = null;
       this.mPixelScrollDelta = 0;
       this.mObservesComposite = false;
-      this.mShowWeekNumber = true;
       this.mToday = false;
       this.mSelected = false;
       this.mExtra = false;
       this.mValue = new Date(); // Default to "today".
       this.mFocused = null;
 
+      let width = 0;
+      // Start loop from 1 as it is needed to get the first month name string
+      // and avoid extra computation of adding one.
+      for (let i = 1; i <= 12; i++) {
+        let dateString = cal.l10n.getDateFmtString(`month.${i}.name`);
+        width = Math.max(dateString.length, width);
+      }
+      this.querySelector(".minimonth-month-name").style.width = `${width + 1}ch`;
+
       this.refreshDisplay();
       if (this.hasAttribute("freebusy")) {
         this._setFreeBusy(this.getAttribute("freebusy") == "true");
       }
-      this.mShowWeekNumber = Services.prefs.getBoolPref(
-        "calendar.view-minimonth.showWeekNumber",
-        true
-      );
-
-      // Add pref observer.
-      Services.prefs.getBranch("").addObserver("calendar.", this.nsIObserver);
 
       // Add event listeners.
       this.addEventListener("click", event => {
-        if (event.button == 0 && event.originalTarget.classList.contains("minimonth-day")) {
+        if (event.button == 0 && event.target.classList.contains("minimonth-day")) {
           this.onDayActivate(event);
         }
       });
 
       this.addEventListener("keypress", event => {
-        if (event.originalTarget.classList.contains("minimonth-day")) {
+        if (event.target.classList.contains("minimonth-day")) {
           if (event.altKey || event.metaKey) {
             return;
           }
@@ -249,7 +258,7 @@
         }
 
         if (deltaView != 0) {
-          const classList = event.originalTarget.classList;
+          const classList = event.target.classList;
 
           if (
             classList.contains("years-forward-button") ||
@@ -409,14 +418,25 @@
 
     // End of calIOperationListener methods.
     // calIObserver methods.
+    calendarsInBatch = new Set();
 
-    onStartBatch(aCalendar) {}
+    onStartBatch(aCalendar) {
+      this.calendarsInBatch.add(aCalendar);
+    }
 
-    onEndBatch(aCalendar) {}
+    onEndBatch(aCalendar) {
+      this.calendarsInBatch.delete(aCalendar);
+    }
 
-    onLoad(aCalendar) {}
+    onLoad(aCalendar) {
+      this.getItems(aCalendar);
+    }
 
     onAddItem(aItem) {
+      if (this.calendarsInBatch.has(aItem.calendar)) {
+        return;
+      }
+
       this.setBusyDaysForItem(aItem, true);
     }
 
@@ -425,6 +445,10 @@
     }
 
     onModifyItem(aNewItem, aOldItem) {
+      if (this.calendarsInBatch.has(aNewItem.calendar)) {
+        return;
+      }
+
       this.setBusyDaysForItem(aOldItem, false);
       this.setBusyDaysForItem(aNewItem, true);
     }
@@ -464,27 +488,8 @@
     onDefaultCalendarChanged(aCalendar) {}
 
     // End calICompositeObserver methods.
-    // nsIObserver methods.
-
-    observe(aSubject, aTopic, aData) {
-      switch (aData) {
-        case "calendar.week.start":
-        case "calendar.view-minimonth.showWeekNumber":
-          delete this.mDayMap; // Days have moved, force a refresh of the grid.
-          this.refreshDisplay();
-          break;
-      }
-    }
-
-    // End nsIObserver methods.
 
     refreshDisplay() {
-      // Find out which should be the first day of the week.
-      this.weekStart = Services.prefs.getIntPref("calendar.week.start", 0);
-      this.mShowWeekNumber = Services.prefs.getBoolPref(
-        "calendar.view-minimonth.showWeekNumber",
-        true
-      );
       if (!this.mValue) {
         this.mValue = new Date();
       }
@@ -560,7 +565,7 @@
         }
       }
 
-      setBooleanAttribute(this._getCalBoxNode(0, 0), "hidden", !this.mShowWeekNumber);
+      this._getCalBoxNode(0, 0).hidden = !this.showWeekNumber;
       for (let column = 1; column < 8; column++) {
         let node = this._getCalBoxNode(0, column);
         node.textContent = dayList[column - 1];
@@ -610,10 +615,12 @@
       }
 
       // Update the month and year title.
-      this.setAttribute("month", aDate.getMonth());
       this.setAttribute("year", aDate.getFullYear());
 
+      let miniMonthName = this.querySelector(".minimonth-month-name");
       let dateString = cal.l10n.getDateFmtString(`month.${aDate.getMonth() + 1}.name`);
+      miniMonthName.setAttribute("value", dateString);
+      miniMonthName.setAttribute("monthIndex", aDate.getMonth());
       this.mReadOnlyHeader.textContent = dateString + " " + aDate.getFullYear();
 
       // Update the calendar.
@@ -633,12 +640,11 @@
 
       this.mDayMap = {};
       let defaultTz = cal.dtz.defaultTimezone;
-      let dateFormatter = new Services.intl.DateTimeFormat(undefined, { dateStyle: "long" });
       for (let k = 1; k < 7; k++) {
         // Set the week number.
         let firstElement = this._getCalBoxNode(k, 0);
-        setBooleanAttribute(firstElement, "hidden", !this.mShowWeekNumber);
-        if (this.mShowWeekNumber) {
+        firstElement.hidden = !this.showWeekNumber;
+        if (this.showWeekNumber) {
           let weekNumber = cal
             .getWeekInfoService()
             .getWeekTitle(cal.dtz.jsDateToDateTime(date, defaultTz));
@@ -684,7 +690,7 @@
           if (aDate.getMonth() == date.getMonth() && aDate.getFullYear() == date.getFullYear()) {
             day.setAttribute("aria-label", date.toLocaleDateString(undefined, { day: "numeric" }));
           } else {
-            day.setAttribute("aria-label", dateFormatter.format(date));
+            day.setAttribute("aria-label", this.dateFormatter.format(date));
           }
 
           day.removeAttribute("busy");
@@ -802,8 +808,7 @@
     updateAccessibleLabel() {
       let label;
       if (this.mValue) {
-        let dateFormatter = new Services.intl.DateTimeFormat(undefined, { dateStyle: "long" });
-        label = dateFormatter.format(this.mValue);
+        label = this.dateFormatter.format(this.mValue);
       } else {
         label = cal.l10n.getCalString("minimonthNoSelectedDate");
       }
@@ -956,7 +961,7 @@
 
     onDayActivate(aEvent) {
       // The associated date might change when setting this.value if month changes.
-      const date = aEvent.originalTarget.date;
+      const date = aEvent.target.date;
       if (this.getAttribute("readonly") != "true") {
         this.value = date;
         this.fireEvent("select");
@@ -976,17 +981,19 @@
       if (this.mObservesComposite) {
         cal.view.getCompositeCalendar(window).removeObserver(this.calICompositeObserver);
       }
-
-      // Remove pref observer.
-      Services.prefs.getBranch("").removeObserver("calendar.", this.nsIObserver);
     }
   }
 
+  XPCOMUtils.defineLazyGetter(
+    CalendarMinimonth.prototype,
+    "dateFormatter",
+    () => new Services.intl.DateTimeFormat(undefined, { dateStyle: "long" })
+  );
+
   MozXULElement.implementCustomInterface(CalendarMinimonth, [
     Ci.calIObserver,
     Ci.calICompositeObserver,
     Ci.calIOperationListener,
-    Ci.nsIObserver,
   ]);
   customElements.define("calendar-minimonth", CalendarMinimonth);
 }
