@@ -2,10 +2,32 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+/* exported makeMemberAttr, makeMemberAttrProperty */
+
 var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
 var { CalAttendee } = ChromeUtils.import("resource:///modules/CalAttendee.jsm");
 var { CalRelation } = ChromeUtils.import("resource:///modules/CalRelation.jsm");
 var { CalAttachment } = ChromeUtils.import("resource:///modules/CalAttachment.jsm");
+var { XPCOMUtils } = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+
+XPCOMUtils.defineLazyModuleGetters(this, {
+  CalAlarm: "resource:///modules/CalAlarm.jsm",
+  CalRecurrenceInfo: "resource:///modules/CalRecurrenceInfo.jsm",
+});
+
+XPCOMUtils.defineLazyServiceGetter(
+  this,
+  "gParserUtils",
+  "@mozilla.org/parserutils;1",
+  "nsIParserUtils"
+);
+
+XPCOMUtils.defineLazyServiceGetter(
+  this,
+  "gTextToHtmlConverter",
+  "@mozilla.org/txttohtmlconv;1",
+  "mozITXTToHTMLConv"
+);
 
 /**
  * calItemBase prototype definition
@@ -48,13 +70,13 @@ calItemBase.prototype = {
     this.wrappedJSObject = this;
     this.mProperties = new Map();
     this.mPropertyParams = {};
-    this.mProperties.set("CREATED", cal.dtz.jsDateToDateTime(new Date()));
+    this.setProperty("CREATED", cal.dtz.jsDateToDateTime(new Date()));
   },
 
   /**
    * @see nsISupports
    */
-  QueryInterface: ChromeUtils.generateQI([Ci.calIItemBase]),
+  QueryInterface: ChromeUtils.generateQI(["calIItemBase"]),
 
   /**
    * @see calIItemBase
@@ -101,7 +123,6 @@ calItemBase.prototype = {
     if (this.mRecurrenceInfo) {
       this.mRecurrenceInfo.onIdChange(uid);
     }
-    return uid;
   },
 
   // attribute calIDateTime recurrenceId;
@@ -110,7 +131,7 @@ calItemBase.prototype = {
   },
   set recurrenceId(rid) {
     this.mHashId = null; // recompute hashId
-    return this.setProperty("RECURRENCE-ID", rid);
+    this.setProperty("RECURRENCE-ID", rid);
   },
 
   // attribute calIRecurrenceInfo recurrenceInfo;
@@ -119,7 +140,7 @@ calItemBase.prototype = {
   },
   set recurrenceInfo(value) {
     this.modify();
-    return (this.mRecurrenceInfo = cal.unwrapInstance(value));
+    this.mRecurrenceInfo = cal.unwrapInstance(value);
   },
 
   // attribute calIItemBase parentItem;
@@ -130,7 +151,7 @@ calItemBase.prototype = {
     if (this.mImmutable) {
       throw Components.Exception("", Cr.NS_ERROR_OBJECT_IS_IMMUTABLE);
     }
-    return (this.mParentItem = cal.unwrapInstance(value));
+    this.mParentItem = cal.unwrapInstance(value);
   },
 
   /**
@@ -243,6 +264,20 @@ calItemBase.prototype = {
     );
   },
 
+  /**
+   * Overridden by CalEvent to indicate the item is an event.
+   */
+  isEvent() {
+    return false;
+  },
+
+  /**
+   * Overridden by CalTodo to indicate the item is a todo.
+   */
+  isTodo() {
+    return false;
+  },
+
   // calIItemBase clone();
   clone() {
     return this.cloneShallow(this.mParentItem);
@@ -335,7 +370,7 @@ calItemBase.prototype = {
     if (aValue && !aValue.timezone.isUTC) {
       aValue = aValue.getInTimezone(cal.dtz.UTC);
     }
-    return (this.mAlarmLastAck = aValue);
+    this.mAlarmLastAck = aValue;
   },
 
   // readonly attribute calIDateTime lastModifiedTime;
@@ -350,6 +385,63 @@ calItemBase.prototype = {
     return this.getProperty("DTSTAMP");
   },
 
+  // attribute AUTF8string descriptionText;
+  get descriptionText() {
+    return this.getProperty("DESCRIPTION");
+  },
+
+  set descriptionText(text) {
+    this.setProperty("DESCRIPTION", text);
+    if (text) {
+      this.setPropertyParameter("DESCRIPTION", "ALTREP", null);
+    } // else: property parameter deleted by setProperty(..., null)
+  },
+
+  // attribute AUTF8string descriptionHTML;
+  get descriptionHTML() {
+    let altrep = this.getPropertyParameter("DESCRIPTION", "ALTREP");
+    if (altrep?.startsWith("data:text/html,")) {
+      try {
+        return decodeURIComponent(altrep.slice("data:text/html,".length));
+      } catch (ex) {
+        console.error(ex);
+      }
+    }
+    // Fallback: Upconvert the plaintext
+    let description = this.getProperty("DESCRIPTION");
+    if (!description) {
+      return null;
+    }
+    let mode = Ci.mozITXTToHTMLConv.kStructPhrase | Ci.mozITXTToHTMLConv.kURLs;
+    description = gTextToHtmlConverter.scanTXT(description, mode);
+    return description.replace(/\r?\n/g, "<br>");
+  },
+
+  set descriptionHTML(html) {
+    if (html) {
+      // Using the same mode as the HTML description editor
+      // in calendar-item-iframe.js
+      let mode =
+        Ci.nsIDocumentEncoder.OutputDropInvisibleBreak |
+        Ci.nsIDocumentEncoder.OutputWrap |
+        Ci.nsIDocumentEncoder.OutputLFLineBreak |
+        Ci.nsIDocumentEncoder.OutputNoScriptContent |
+        Ci.nsIDocumentEncoder.OutputNoFramesContent |
+        Ci.nsIDocumentEncoder.OutputBodyOnly;
+      let text = gParserUtils.convertToPlainText(html, mode, 80);
+      this.setProperty("DESCRIPTION", text);
+      if (text) {
+        this.setPropertyParameter(
+          "DESCRIPTION",
+          "ALTREP",
+          "data:text/html," + encodeURIComponent(html)
+        );
+      } // else: can't set a property parameter if the property is empty
+    } else {
+      this.deleteProperty("DESCRIPTION");
+    }
+  },
+
   // readonly attribute nsIJSEnumerator properties;
   get properties() {
     let properties = this.mProperties;
@@ -387,6 +479,9 @@ calItemBase.prototype = {
     aName = aName.toUpperCase();
     if (aValue || !isNaN(parseInt(aValue, 10))) {
       this.mProperties.set(aName, aValue);
+      if (!(aName in this.mPropertyParams)) {
+        this.mPropertyParams[aName] = {};
+      }
     } else {
       this.deleteProperty(aName);
     }
@@ -453,32 +548,16 @@ calItemBase.prototype = {
     return aParamValue;
   },
 
-  // nsISimpleEnumerator getParameterEnumerator(in AString aPropertyName);
-  getParameterEnumerator(aPropName) {
+  // Array<AString> getParameterNames(in AString aPropertyName);
+  getParameterNames(aPropName) {
     let propName = aPropName.toUpperCase();
     if (!(propName in this.mPropertyParams)) {
+      if (this.mIsProxy) {
+        return this.mParentItem.getParameterNames(aPropName);
+      }
       throw new Error("Property " + aPropName + " not set");
     }
-    let parameters = this.mPropertyParams[propName];
-    return {
-      // nsISimpleEnumerator
-      QueryInterface: ChromeUtils.generateQI([Ci.nsISimpleEnumerator]),
-
-      mParamNames: Object.keys(parameters),
-      hasMoreElements() {
-        return this.mParamNames.length > 0;
-      },
-
-      getNext() {
-        let paramName = this.mParamNames.pop();
-        return {
-          // nsIProperty
-          QueryInterface: ChromeUtils.generateQI([Ci.nsIProperty]),
-          name: paramName,
-          value: parameters[paramName],
-        };
-      },
-    };
+    return Object.keys(this.mPropertyParams[propName]);
   },
 
   // void getAttendees(out PRUint32 count,
@@ -855,7 +934,7 @@ calItemBase.prototype = {
         ritem.icalProperty = recprop;
 
         if (!rec) {
-          rec = cal.createRecurrenceInfo(this);
+          rec = new CalRecurrenceInfo(this);
         }
         rec.appendRecurrenceItem(ritem);
       }
@@ -864,7 +943,7 @@ calItemBase.prototype = {
 
     this.mAlarms = []; // don't inherit anything from parent
     for (let alarmComp of cal.iterate.icalSubcomponent(icalcomp, "VALARM")) {
-      let alarm = cal.createAlarm();
+      let alarm = new CalAlarm();
       try {
         alarm.icalComponent = alarmComp;
         this.addAlarm(alarm, true);
@@ -931,7 +1010,7 @@ calItemBase.prototype = {
     return gen ? parseInt(gen, 10) : 0;
   },
   set generation(aValue) {
-    return this.setProperty("X-MOZ-GENERATION", String(aValue));
+    this.setProperty("X-MOZ-GENERATION", String(aValue));
   },
 
   /**
@@ -1055,40 +1134,54 @@ calItemBase.prototype = {
   },
 };
 
-makeMemberAttr(calItemBase, "CREATED", null, "creationDate", true);
-makeMemberAttr(calItemBase, "SUMMARY", null, "title", true);
-makeMemberAttr(calItemBase, "PRIORITY", 0, "priority", true);
-makeMemberAttr(calItemBase, "CLASS", "PUBLIC", "privacy", true);
-makeMemberAttr(calItemBase, "STATUS", null, "status", true);
-makeMemberAttr(calItemBase, "ALARMTIME", null, "alarmTime", true);
-
-makeMemberAttr(calItemBase, "mProperties", null, "properties");
+makeMemberAttrProperty(calItemBase, "CREATED", "creationDate");
+makeMemberAttrProperty(calItemBase, "SUMMARY", "title");
+makeMemberAttrProperty(calItemBase, "PRIORITY", "priority");
+makeMemberAttrProperty(calItemBase, "CLASS", "privacy");
+makeMemberAttrProperty(calItemBase, "STATUS", "status");
+makeMemberAttrProperty(calItemBase, "ALARMTIME", "alarmTime");
 
 /**
- * Helper function to add a member attribute on the given prototype
+ * Adds a member attribute to the given prototype.
  *
- * @param ctor          The constructor function of the prototype
- * @param varname       The local variable name to get/set, or the property in
- *                        case asProperty is true.
- * @param dflt          The default value in case none is set
- * @param attr          The attribute name to be used
- * @param asProperty    If true, getProperty will be used to get/set the
- *                        member.
+ * @param {Function} ctor       The constructor function of the prototype.
+ * @param {string} varname      The variable name to get/set.
+ * @param {string} attr         The attribute name to be used.
+ * @param {*} dflt              The default value in case none is set.
  */
-function makeMemberAttr(ctor, varname, dflt, attr, asProperty) {
-  // XXX handle defaults!
+function makeMemberAttr(ctor, varname, attr, dflt) {
   let getter = function() {
-    if (asProperty) {
-      return this.getProperty(varname);
-    }
-    return varname in this ? this[varname] : undefined;
+    return varname in this ? this[varname] : dflt;
   };
   let setter = function(value) {
     this.modify();
-    if (asProperty) {
-      return this.setProperty(varname, value);
-    }
-    return (this[varname] = value);
+    this[varname] = value;
+    return value;
+  };
+
+  ctor.prototype.__defineGetter__(attr, getter);
+  ctor.prototype.__defineSetter__(attr, setter);
+}
+
+/**
+ * Adds a member attribute to the given prototype, using `getProperty` and
+ * `setProperty` for access.
+ *
+ * Default values are not handled here, but instead are set in constructors,
+ * which makes it possible to e.g. iterate through `mProperties` when cloning
+ * an object.
+ *
+ * @param {Function} ctor       The constructor function of the prototype.
+ * @param {string} name         The property name to get/set.
+ * @param {string} attr         The attribute name to be used.
+ */
+function makeMemberAttrProperty(ctor, name, attr) {
+  let getter = function() {
+    return this.getProperty(name);
+  };
+  let setter = function(value) {
+    this.modify();
+    return this.setProperty(name, value);
   };
   ctor.prototype.__defineGetter__(attr, getter);
   ctor.prototype.__defineSetter__(attr, setter);
