@@ -2,6 +2,15 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+var { XPCOMUtils } = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+
+XPCOMUtils.defineLazyModuleGetters(this, {
+  CalAlarm: "resource:///modules/CalAlarm.jsm",
+  CalAttachment: "resource:///modules/CalAttachment.jsm",
+  CalAttendee: "resource:///modules/CalAttendee.jsm",
+  CalTodo: "resource:///modules/CalTodo.jsm",
+});
+
 function run_test() {
   // Initialize the floating timezone without actually starting the service.
   cal.getTimezoneService().floating; // eslint-disable-line no-unused-expressions
@@ -25,7 +34,7 @@ function run_test() {
 
 function test_initial_creation() {
   dump("Testing initial creation...");
-  let alarm = cal.createAlarm();
+  let alarm = new CalAlarm();
 
   let passed;
   try {
@@ -43,7 +52,7 @@ function test_initial_creation() {
 
 function test_display_alarm() {
   dump("Testing DISPLAY alarms...");
-  let alarm = cal.createAlarm();
+  let alarm = new CalAlarm();
   // Set ACTION to DISPLAY, make sure this was not rejected
   alarm.action = "DISPLAY";
   equal(alarm.action, "DISPLAY");
@@ -57,7 +66,7 @@ function test_display_alarm() {
   equal(alarm.summary, null);
 
   // No attendees allowed
-  let attendee = cal.createAttendee();
+  let attendee = new CalAttendee();
   attendee.id = "mailto:horst";
 
   throws(() => {
@@ -67,7 +76,7 @@ function test_display_alarm() {
 
   throws(() => {
     // DISPLAY alarm should not be able to save attachment
-    alarm.addAttachment(cal.createAttachment());
+    alarm.addAttachment(new CalAttachment());
   }, /Alarm type DISPLAY may not have attachments/);
 
   dump("Done\n");
@@ -75,7 +84,7 @@ function test_display_alarm() {
 
 function test_email_alarm() {
   dump("Testing EMAIL alarms...");
-  let alarm = cal.createAlarm();
+  let alarm = new CalAlarm();
   // Set ACTION to DISPLAY, make sure this was not rejected
   alarm.action = "EMAIL";
   equal(alarm.action, "EMAIL");
@@ -93,9 +102,9 @@ function test_email_alarm() {
   alarm.offset = cal.createDuration();
 
   // Check for at least one attendee
-  let attendee1 = cal.createAttendee();
+  let attendee1 = new CalAttendee();
   attendee1.id = "mailto:horst";
-  let attendee2 = cal.createAttendee();
+  let attendee2 = new CalAttendee();
   attendee2.id = "mailto:gustav";
 
   equal(alarm.getAttendees().length, 0);
@@ -106,8 +115,8 @@ function test_email_alarm() {
   alarm.addAttendee(attendee1);
   let addedAttendees = alarm.getAttendees();
   equal(addedAttendees.length, 2);
-  equal(addedAttendees[0], attendee2);
-  equal(addedAttendees[1], attendee1);
+  equal(addedAttendees[0].wrappedJSObject, attendee2);
+  equal(addedAttendees[1].wrappedJSObject, attendee1);
 
   ok(!!alarm.icalComponent.serializeToICS().match(/mailto:horst/));
   ok(!!alarm.icalComponent.serializeToICS().match(/mailto:gustav/));
@@ -137,7 +146,7 @@ function test_email_alarm() {
 
 function test_audio_alarm() {
   dump("Testing AUDIO alarms...");
-  let alarm = cal.createAlarm();
+  let alarm = new CalAlarm();
   alarm.related = Ci.calIAlarm.ALARM_RELATED_ABSOLUTE;
   alarm.alarmDate = cal.createDateTime();
   // Set ACTION to AUDIO, make sure this was not rejected
@@ -153,7 +162,7 @@ function test_audio_alarm() {
   equal(alarm.summary, null);
 
   // No attendees allowed
-  let attendee = cal.createAttendee();
+  let attendee = new CalAttendee();
   attendee.id = "mailto:horst";
 
   try {
@@ -164,23 +173,23 @@ function test_audio_alarm() {
   }
 
   // Test attachments
-  let sound = cal.createAttachment();
+  let sound = new CalAttachment();
   sound.uri = Services.io.newURI("file:///sound.wav");
-  let sound2 = cal.createAttachment();
+  let sound2 = new CalAttachment();
   sound2.uri = Services.io.newURI("file:///sound2.wav");
 
   // Adding an attachment should work
   alarm.addAttachment(sound);
   let addedAttachments = alarm.getAttachments();
   equal(addedAttachments.length, 1);
-  equal(addedAttachments[0], sound);
+  equal(addedAttachments[0].wrappedJSObject, sound);
   ok(alarm.icalString.includes("ATTACH:file:///sound.wav"));
 
   // Adding twice shouldn't change anything
   alarm.addAttachment(sound);
   addedAttachments = alarm.getAttachments();
   equal(addedAttachments.length, 1);
-  equal(addedAttachments[0], sound);
+  equal(addedAttachments[0].wrappedJSObject, sound);
 
   try {
     alarm.addAttachment(sound2);
@@ -218,7 +227,7 @@ function test_audio_alarm() {
 
 function test_custom_alarm() {
   dump("Testing X-SMS (custom) alarms...");
-  let alarm = cal.createAlarm();
+  let alarm = new CalAlarm();
   // Set ACTION to a custom value, make sure this was not rejected
   alarm.action = "X-SMS";
   equal(alarm.action, "X-SMS");
@@ -232,9 +241,9 @@ function test_custom_alarm() {
   equal(alarm.summary, "summary");
 
   // Test for attendees
-  let attendee1 = cal.createAttendee();
+  let attendee1 = new CalAttendee();
   attendee1.id = "mailto:horst";
-  let attendee2 = cal.createAttendee();
+  let attendee2 = new CalAttendee();
   attendee2.id = "mailto:gustav";
 
   equal(alarm.getAttendees().length, 0);
@@ -252,9 +261,9 @@ function test_custom_alarm() {
   equal(alarm.getAttendees().length, 0);
 
   // Test for attachments
-  let attach1 = cal.createAttachment();
+  let attach1 = new CalAttachment();
   attach1.uri = Services.io.newURI("file:///example.txt");
-  let attach2 = cal.createAttachment();
+  let attach2 = new CalAttachment();
   attach2.uri = Services.io.newURI("file:///example2.txt");
 
   alarm.addAttachment(attach1);
@@ -262,8 +271,8 @@ function test_custom_alarm() {
 
   let addedAttachments = alarm.getAttachments();
   equal(addedAttachments.length, 2);
-  equal(addedAttachments[0], attach1);
-  equal(addedAttachments[1], attach2);
+  equal(addedAttachments[0].wrappedJSObject, attach1);
+  equal(addedAttachments[1].wrappedJSObject, attach2);
 
   alarm.deleteAttachment(attach1);
   addedAttachments = alarm.getAttachments();
@@ -277,7 +286,7 @@ function test_custom_alarm() {
 // Check if any combination of REPEAT and DURATION work as expected.
 function test_repeat() {
   dump("Testing REPEAT and DURATION properties...");
-  let alarm = cal.createAlarm();
+  let alarm = new CalAlarm();
 
   // Check initial value
   equal(alarm.repeat, 0);
@@ -309,7 +318,7 @@ function test_repeat() {
   equal(alarm.repeatOffset, null);
 
   // Check repeatDate
-  alarm = cal.createAlarm();
+  alarm = new CalAlarm();
   alarm.related = Ci.calIAlarm.ALARM_RELATED_ABSOLUTE;
   alarm.alarmDate = cal.createDateTime();
   alarm.repeat = 1;
@@ -325,7 +334,7 @@ function test_repeat() {
 
 function test_xprop() {
   dump("Testing X-Props...");
-  let alarm = cal.createAlarm();
+  let alarm = new CalAlarm();
   alarm.setProperty("X-PROP", "X-VALUE");
   ok(alarm.hasProperty("X-PROP"));
   equal(alarm.getProperty("X-PROP"), "X-VALUE");
@@ -351,7 +360,7 @@ function test_dates() {
   dump("Testing alarm dates...");
   let passed;
   // Initial value
-  let alarm = cal.createAlarm();
+  let alarm = new CalAlarm();
   equal(alarm.alarmDate, null);
   equal(alarm.offset, null);
 
@@ -411,7 +420,7 @@ var clonePropMap = {
 
 function test_immutable() {
   dump("Testing immutable alarms...");
-  let alarm = cal.createAlarm();
+  let alarm = new CalAlarm();
   // Set up each attribute
   for (let prop in propMap) {
     alarm[prop] = propMap[prop];
@@ -420,7 +429,7 @@ function test_immutable() {
   // Set up some extra props
   alarm.setProperty("X-FOO", "X-VAL");
   alarm.setProperty("X-DATEPROP", cal.createDateTime());
-  alarm.addAttendee(cal.createAttendee());
+  alarm.addAttendee(new CalAttendee());
 
   // Initial checks
   ok(alarm.isMutable);
@@ -456,7 +465,7 @@ function test_immutable() {
 
 function test_clone() {
   dump("Testing cloning alarms...");
-  let alarm = cal.createAlarm();
+  let alarm = new CalAlarm();
   // Set up each attribute
   for (let prop in propMap) {
     alarm[prop] = propMap[prop];
@@ -465,7 +474,7 @@ function test_clone() {
   // Set up some extra props
   alarm.setProperty("X-FOO", "X-VAL");
   alarm.setProperty("X-DATEPROP", cal.createDateTime());
-  alarm.addAttendee(cal.createAttendee());
+  alarm.addAttendee(new CalAttendee());
 
   // Make a copy
   let newAlarm = alarm.clone();
@@ -520,7 +529,7 @@ function test_clone() {
 
 function test_serialize() {
   // most checks done by other tests, these don't fit into categories
-  let alarm = cal.createAlarm();
+  let alarm = new CalAlarm();
   let srv = cal.getIcsService();
 
   throws(
@@ -637,7 +646,7 @@ function test_serialize() {
 function test_strings() {
   // Serializing the string shouldn't throw, but we don't really care about
   // the string itself.
-  let alarm = cal.createAlarm();
+  let alarm = new CalAlarm();
   alarm.action = "DISPLAY";
   alarm.related = Ci.calIAlarm.ALARM_RELATED_ABSOLUTE;
   alarm.alarmDate = cal.createDateTime();
@@ -646,12 +655,12 @@ function test_strings() {
   alarm.related = Ci.calIAlarm.ALARM_RELATED_START;
   alarm.offset = cal.createDuration();
   alarm.toString();
-  alarm.toString(cal.createTodo());
+  alarm.toString(new CalTodo());
 
   alarm.related = Ci.calIAlarm.ALARM_RELATED_END;
   alarm.offset = cal.createDuration();
   alarm.toString();
-  alarm.toString(cal.createTodo());
+  alarm.toString(new CalTodo());
 
   alarm.offset = cal.createDuration("P1D");
   alarm.toString();
