@@ -6,6 +6,10 @@
 const URL1 = MAIN_DOMAIN + "navigate-first.html";
 const URL2 = MAIN_DOMAIN + "navigate-second.html";
 
+const { PromptTestUtils } = ChromeUtils.import(
+  "resource://testing-common/PromptTestUtils.jsm"
+);
+
 var isE10s = Services.appinfo.browserTabsRemoteAutostart;
 
 SpecialPowers.pushPrefEnv({
@@ -71,11 +75,6 @@ function assertEvent(event, data) {
         "navigate",
         "Then once the second doc is loaded, we get the navigate event"
       );
-      is(
-        data.readyState,
-        "complete",
-        "navigate is emitted only once the document is fully loaded"
-      );
       break;
     case 9:
       is(event, "tabNavigated", "Finally, the receive the client event");
@@ -88,22 +87,6 @@ function assertEvent(event, data) {
   }
 }
 
-function waitForOnBeforeUnloadDialog(browser, callback) {
-  browser.addEventListener(
-    "DOMWillOpenModalDialog",
-    async function(event) {
-      const stack = browser.parentNode;
-      const dialogs = stack.getElementsByTagName("tabmodalprompt");
-      await waitUntil(() => dialogs[0]);
-      const { button0, button1 } = browser.tabModalPromptBox.getPrompt(
-        dialogs[0]
-      ).ui;
-      callback(button0, button1);
-    },
-    { capture: true, once: true }
-  );
-}
-
 var httpObserver = function(subject, topic, state) {
   const channel = subject.QueryInterface(Ci.nsIHttpChannel);
   const url = channel.URI.spec;
@@ -113,18 +96,27 @@ var httpObserver = function(subject, topic, state) {
   }
 };
 Services.obs.addObserver(httpObserver, "http-on-modify-request");
+registerCleanupFunction(() => {
+  Services.obs.removeObserver(httpObserver, "http-on-modify-request");
+});
 
 function onMessage({ data }) {
   assertEvent(data.event, data.data);
 }
 
 async function connectAndAttachTab(tab) {
-  const target = await TargetFactory.forTab(tab);
-  await target.attach();
-  const actorID = target.targetForm.actor;
+  const target = await createAndAttachTargetForTab(tab);
+  const actorID = target.actorID;
   target.on("tabNavigated", function(packet) {
     assertEvent("tabNavigated", packet);
   });
+  // In order to listen to internal will-navigate/navigate events
+  target.on("will-navigate", function(data) {
+    assertEvent("will-navigate", {
+      newURI: data.url,
+    });
+  });
+  target.on("navigate", () => assertEvent("navigate"));
   return { target, actorID };
 }
 
@@ -133,10 +125,12 @@ add_task(async function() {
   const browser = await addTab(URL1);
 
   // Listen for alert() call being made in navigate-first during unload
-  waitForOnBeforeUnloadDialog(browser, function(btnLeave, btnStay) {
+  const beforeUnloadPromise = PromptTestUtils.handleNextPrompt(
+    browser,
+    { modalType: Services.prompt.MODAL_TYPE_CONTENT, promptType: "confirmEx" },
+    { buttonNumClick: 0 }
+  ).then(() => {
     assertEvent("unload-dialog");
-    // accept to quit this page to another
-    btnLeave.click();
   });
 
   // Listen for messages sent by the content task
@@ -145,27 +139,6 @@ add_task(async function() {
   const tab = gBrowser.getTabForBrowser(browser);
   const { target, actorID } = await connectAndAttachTab(tab);
   await ContentTask.spawn(browser, [actorID], async function(actorId) {
-    const { require } = ChromeUtils.import(
-      "resource://devtools/shared/Loader.jsm"
-    );
-    const { DevToolsServer } = require("devtools/server/devtools-server");
-    const EventEmitter = require("devtools/shared/event-emitter");
-
-    // !Hack! Retrieve a server side object, the FrameTargetActor instance
-    const targetActor = DevToolsServer.searchAllConnectionsForActor(actorId);
-    // In order to listen to internal will-navigate/navigate events
-    EventEmitter.on(targetActor, "will-navigate", function(data) {
-      sendSyncMessage("devtools-test:event", {
-        event: "will-navigate",
-        data: { newURI: data.newURI },
-      });
-    });
-    EventEmitter.on(targetActor, "navigate", function(data) {
-      sendSyncMessage("devtools-test:event", {
-        event: "navigate",
-        data: { readyState: content.document.readyState },
-      });
-    });
     // Forward DOMContentLoaded and load events
     addEventListener(
       "DOMContentLoaded",
@@ -196,7 +169,8 @@ add_task(async function() {
   const onBrowserLoaded = BrowserTestUtils.browserLoaded(
     gBrowser.selectedBrowser
   );
-  await BrowserTestUtils.loadURI(gBrowser.selectedBrowser, URL2);
+  BrowserTestUtils.loadURI(gBrowser.selectedBrowser, URL2);
+  await beforeUnloadPromise;
   await onBrowserLoaded;
 
   // Wait for all events to be received
