@@ -5,12 +5,12 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsString.h"
-#include "nsDataHashtable.h"
+#include "nsTHashMap.h"
 
 namespace mozilla {
 namespace net {
 
-typedef nsDataHashtable<nsCStringHashKey, nsCString> AddrMapType;
+using AddrMapType = nsTHashMap<nsCStringHashKey, nsCString>;
 
 nsresult DoListAddresses(AddrMapType& aAddrMap);
 
