@@ -35,7 +35,7 @@ class AsyncApplyBufferingPolicyEvent final : public Runnable {
   explicit AsyncApplyBufferingPolicyEvent(nsAsyncStreamCopier* aCopier)
       : mozilla::Runnable("AsyncApplyBufferingPolicyEvent"),
         mCopier(aCopier),
-        mTarget(GetCurrentThreadEventTarget()) {}
+        mTarget(GetCurrentEventTarget()) {}
 
   NS_IMETHOD Run() override {
     nsresult rv = mCopier->ApplyBufferingPolicy();
@@ -64,14 +64,7 @@ class AsyncApplyBufferingPolicyEvent final : public Runnable {
 //-----------------------------------------------------------------------------
 
 nsAsyncStreamCopier::nsAsyncStreamCopier()
-    : mLock("nsAsyncStreamCopier.mLock"),
-      mMode(NS_ASYNCCOPY_VIA_READSEGMENTS),
-      mChunkSize(nsIOService::gDefaultSegmentSize),
-      mStatus(NS_OK),
-      mIsPending(false),
-      mCloseSource{false},
-      mCloseSink{false},
-      mShouldSniffBuffering(false) {
+    : mChunkSize(nsIOService::gDefaultSegmentSize) {
   LOG(("Creating nsAsyncStreamCopier @%p\n", this));
 }
 
