@@ -35,10 +35,10 @@ namespace mozilla {
 namespace net {
 
 Http2Stream::Http2Stream(nsAHttpTransaction* httpTransaction,
-                         Http2Session* session, int32_t priority,
-                         uint64_t windowId)
+                         Http2Session* session, int32_t priority, uint64_t bcId)
     : mStreamID(0),
-      mSession(session),
+      mSession(
+          do_GetWeakReference(static_cast<nsISupportsWeakReference*>(session))),
       mSegmentReader(nullptr),
       mSegmentWriter(nullptr),
       mUpstreamState(GENERATING_HEADERS),
@@ -48,7 +48,7 @@ Http2Stream::Http2Stream(nsAHttpTransaction* httpTransaction,
       mAllHeadersReceived(0),
       mQueued(0),
       mSocketTransport(session->SocketTransport()),
-      mCurrentForegroundTabOuterContentWindowId(windowId),
+      mCurrentTopBrowsingContextId(bcId),
       mTransactionTabId(0),
       mTransaction(httpTransaction),
       mChunkSize(session->SendingChunkSize()),
@@ -104,11 +104,13 @@ Http2Stream::Http2Stream(nsAHttpTransaction* httpTransaction,
   SetPriority(static_cast<uint32_t>(httpPriority));
 
   if (trans) {
-    mTransactionTabId = trans->TopLevelOuterContentWindowId();
+    mTransactionTabId = trans->TopBrowsingContextId();
   }
 }
 
 Http2Stream::~Http2Stream() {
+  MOZ_DIAGNOSTIC_ASSERT(OnSocketThread());
+
   ClearPushSource();
   ClearTransactionsBlockedOnTunnel();
   mStreamID = Http2Session::kDeadStreamID;
@@ -116,6 +118,12 @@ Http2Stream::~Http2Stream() {
   LOG3(("Http2Stream::~Http2Stream %p", this));
 }
 
+Http2Session* Http2Stream::Session() {
+  RefPtr<Http2Session> session = do_QueryReferent(mSession);
+  MOZ_RELEASE_ASSERT(session);
+  return session.get();
+}
+
 void Http2Stream::ClearPushSource() {
   if (mPushSource) {
     mPushSource->SetConsumerStream(nullptr);
@@ -132,6 +140,8 @@ nsresult Http2Stream::ReadSegments(nsAHttpSegmentReader* reader, uint32_t count,
                                    uint32_t* countRead) {
   LOG3(("Http2Stream %p ReadSegments reader=%p count=%d state=%x", this, reader,
         count, mUpstreamState));
+  // Reader is nullptr when this is a push stream.
+  MOZ_DIAGNOSTIC_ASSERT(!reader || (reader == Session()));
 
   MOZ_ASSERT(OnSocketThread(), "not on socket thread");
 
@@ -171,8 +181,9 @@ nsresult Http2Stream::ReadSegments(nsAHttpSegmentReader* reader, uint32_t count,
       // Check to see if the transaction's request could be written out now.
       // If not, mark the stream for callback when writing can proceed.
       if (NS_SUCCEEDED(rv) && mUpstreamState == GENERATING_HEADERS &&
-          !mRequestHeadersDone)
-        mSession->TransactionHasDataToWrite(this);
+          !mRequestHeadersDone) {
+        Session()->TransactionHasDataToWrite(this);
+      }
 
       // mTxinlineFrameUsed represents any queued un-sent frame. It might
       // be 0 if there is no such frame, which is not a gurantee that we
@@ -228,7 +239,7 @@ nsresult Http2Stream::ReadSegments(nsAHttpSegmentReader* reader, uint32_t count,
         } else {
           GenerateDataFrameHeader(0, true);
           ChangeState(SENDING_FIN_STREAM);
-          mSession->TransactionHasDataToWrite(this);
+          Session()->TransactionHasDataToWrite(this);
           rv = NS_BASE_STREAM_WOULD_BLOCK;
         }
       }
@@ -450,7 +461,7 @@ nsresult Http2Stream::ParseHttpRequestHeaders(const char* buf, uint32_t avail,
   mSocketTransport->GetOriginAttributes(&originAttributes);
 
   CreatePushHashKey(nsDependentCString(head->IsHTTPS() ? "https" : "http"),
-                    authorityHeader, originAttributes, mSession->Serial(),
+                    authorityHeader, originAttributes, Session()->Serial(),
                     requestURI, mOrigin, hashkey);
 
   // check the push cache for GET
@@ -471,14 +482,14 @@ nsresult Http2Stream::ParseHttpRequestHeaders(const char* buf, uint32_t avail,
     nsHttpTransaction* trans = mTransaction->QueryHttpTransaction();
     if (trans && (pushedStreamWrapper = trans->TakePushedStream()) &&
         (pushedStream = pushedStreamWrapper->GetStream())) {
-      if (pushedStream->mSession == mSession) {
+      if (pushedStream->Session() == Session()) {
         LOG3(("Pushed Stream match based on OnPush correlation %p",
               pushedStream));
       } else {
         LOG3(("Pushed Stream match failed due to stream mismatch %p %" PRId64
               " %" PRId64 "\n",
-              pushedStream, pushedStream->mSession->Serial(),
-              mSession->Serial()));
+              pushedStream, pushedStream->Session()->Serial(),
+              Session()->Serial()));
         pushedStream->OnPushFailed();
         pushedStream = nullptr;
       }
@@ -494,7 +505,7 @@ nsresult Http2Stream::ParseHttpRequestHeaders(const char* buf, uint32_t avail,
     LOG3(
         ("Pushed Stream Lookup "
          "session=%p key=%s requestcontext=%p cache=%p hit=%p\n",
-         mSession, hashkey.get(), requestContext, cache, pushedStream));
+         Session(), hashkey.get(), requestContext, cache, pushedStream));
 
     if (pushedStream) {
       LOG3(("Pushed Stream Match located %p id=0x%X key=%s\n", pushedStream,
@@ -506,13 +517,13 @@ nsresult Http2Stream::ParseHttpRequestHeaders(const char* buf, uint32_t avail,
 
       // There is probably pushed data buffered so trigger a read manually
       // as we can't rely on future network events to do it
-      mSession->ConnectPushedStream(this);
+      Session()->ConnectPushedStream(this);
       mOpenGenerated = 1;
 
       // if the "mother stream" had TRR, this one is a TRR stream too!
       RefPtr<nsHttpConnectionInfo> ci(Transaction()->ConnectionInfo());
       if (ci && ci->GetIsTrrServiceChannel()) {
-        mSession->IncrementTrrCounter();
+        Session()->IncrementTrrCounter();
       }
 
       return NS_OK;
@@ -526,7 +537,7 @@ nsresult Http2Stream::GenerateOpen() {
   // It is now OK to assign a streamID that we are assured will
   // be monotonically increasing amongst new streams on this
   // session
-  mStreamID = mSession->RegisterStreamID(this);
+  mStreamID = Session()->RegisterStreamID(this);
   MOZ_ASSERT(mStreamID & 1, "Http2 Stream Channel ID must be odd");
   MOZ_ASSERT(!mOpenGenerated);
 
@@ -536,7 +547,7 @@ nsresult Http2Stream::GenerateOpen() {
   nsAutoCString requestURI;
   head->RequestURI(requestURI);
   LOG3(("Http2Stream %p Stream ID 0x%X [session=%p] for URI %s\n", this,
-        mStreamID, mSession, requestURI.get()));
+        mStreamID, Session(), requestURI.get()));
 
   if (mStreamID >= 0x80000000) {
     // streamID must fit in 31 bits. Evading This is theoretically possible
@@ -598,12 +609,12 @@ nsresult Http2Stream::GenerateOpen() {
     useSimpleConnect = false;
     protocol.AppendLiteral("websocket");
   }
-  rv = mSession->Compressor()->EncodeHeaderBlock(
+  rv = Session()->Compressor()->EncodeHeaderBlock(
       mFlatHttpRequestHeaders, method, path, authorityHeader, scheme, protocol,
       useSimpleConnect, compressedData);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  int64_t clVal = mSession->Compressor()->GetParsedContentLength();
+  int64_t clVal = Session()->Compressor()->GetParsedContentLength();
   if (clVal != -1) {
     mRequestBodyLenRemaining = clVal;
   }
@@ -688,11 +699,11 @@ nsresult Http2Stream::GenerateOpen() {
     }
     dataLength -= frameLen;
 
-    mSession->CreateFrameHeader(mTxInlineFrame.get() + outputOffset,
-                                frameLen + (idx ? 0 : 5),
-                                (idx) ? Http2Session::FRAME_TYPE_CONTINUATION
-                                      : Http2Session::FRAME_TYPE_HEADERS,
-                                flags, mStreamID);
+    Session()->CreateFrameHeader(mTxInlineFrame.get() + outputOffset,
+                                 frameLen + (idx ? 0 : 5),
+                                 (idx) ? Http2Session::FRAME_TYPE_CONTINUATION
+                                       : Http2Session::FRAME_TYPE_HEADERS,
+                                 flags, mStreamID);
     outputOffset += Http2Session::kFrameHeaderBytes;
 
     if (!idx) {
@@ -757,8 +768,8 @@ void Http2Stream::AdjustInitialWindow() {
                ? (trans->InitialRwin() - mClientReceiveWindow)
                : 0;
   } else {
-    MOZ_ASSERT(mSession->InitialRwin() >= mClientReceiveWindow);
-    bump = mSession->InitialRwin() - mClientReceiveWindow;
+    MOZ_ASSERT(Session()->InitialRwin() >= mClientReceiveWindow);
+    bump = Session()->InitialRwin() - mClientReceiveWindow;
   }
 
   LOG3(("AdjustInitialwindow increased flow control window %p 0x%X %u\n", this,
@@ -773,8 +784,8 @@ void Http2Stream::AdjustInitialWindow() {
   uint8_t* packet = mTxInlineFrame.get() + mTxInlineFrameUsed;
   mTxInlineFrameUsed += Http2Session::kFrameHeaderBytes + 4;
 
-  mSession->CreateFrameHeader(packet, 4, Http2Session::FRAME_TYPE_WINDOW_UPDATE,
-                              0, stream->mStreamID);
+  Session()->CreateFrameHeader(
+      packet, 4, Http2Session::FRAME_TYPE_WINDOW_UPDATE, 0, stream->mStreamID);
 
   mClientReceiveWindow += bump;
   bump = PR_htonl(bump);
@@ -802,8 +813,8 @@ void Http2Stream::AdjustPushedPriority() {
   uint8_t* packet = mTxInlineFrame.get() + mTxInlineFrameUsed;
   mTxInlineFrameUsed += Http2Session::kFrameHeaderBytes + 5;
 
-  mSession->CreateFrameHeader(packet, 5, Http2Session::FRAME_TYPE_PRIORITY, 0,
-                              mPushSource->mStreamID);
+  Session()->CreateFrameHeader(packet, 5, Http2Session::FRAME_TYPE_PRIORITY, 0,
+                               mPushSource->mStreamID);
 
   mPushSource->SetPriorityDependency(mPriority, mPriorityDependency);
   uint32_t wireDep = PR_htonl(mPriorityDependency);
@@ -844,9 +855,10 @@ void Http2Stream::UpdateTransportSendEvents(uint32_t count) {
     mSocketTransport->SetSendBufferSize(bufferSize);
   }
 
-  if (mUpstreamState != SENDING_FIN_STREAM)
+  if (mUpstreamState != SENDING_FIN_STREAM) {
     mTransaction->OnTransportStatus(mSocketTransport, NS_NET_STATUS_SENDING_TO,
                                     mTotalSent);
+  }
 
   if (!mSentWaitingFor && !mRequestBodyLenRemaining) {
     mSentWaitingFor = 1;
@@ -901,22 +913,23 @@ nsresult Http2Stream::TransmitFrame(const char* buf, uint32_t* countUsed,
 
   if (rv == NS_BASE_STREAM_WOULD_BLOCK) {
     MOZ_ASSERT(!forceCommitment, "forceCommitment with WOULD_BLOCK");
-    mSession->TransactionHasDataToWrite(this);
+    Session()->TransactionHasDataToWrite(this);
   }
-  if (NS_FAILED(rv))  // this will include WOULD_BLOCK
+  if (NS_FAILED(rv)) {  // this will include WOULD_BLOCK
     return rv;
+  }
 
   // This function calls mSegmentReader->OnReadSegment to report the actual
   // http/2 bytes through to the session object and then the HttpConnection
   // which calls the socket write function. It will accept all of the inline and
   // stream data because of the above 'commitment' even if it has to buffer
 
-  rv = mSession->BufferOutput(reinterpret_cast<char*>(mTxInlineFrame.get()),
-                              mTxInlineFrameUsed, &transmittedCount);
+  rv = Session()->BufferOutput(reinterpret_cast<char*>(mTxInlineFrame.get()),
+                               mTxInlineFrameUsed, &transmittedCount);
   LOG3(
       ("Http2Stream::TransmitFrame for inline BufferOutput session=%p "
        "stream=%p result %" PRIx32 " len=%d",
-       mSession, this, static_cast<uint32_t>(rv), transmittedCount));
+       Session(), this, static_cast<uint32_t>(rv), transmittedCount));
 
   MOZ_ASSERT(rv != NS_BASE_STREAM_WOULD_BLOCK,
              "inconsistent inline commitment result");
@@ -926,7 +939,7 @@ nsresult Http2Stream::TransmitFrame(const char* buf, uint32_t* countUsed,
   MOZ_ASSERT(transmittedCount == mTxInlineFrameUsed,
              "inconsistent inline commitment count");
 
-  Http2Session::LogIO(mSession, this, "Writing from Inline Buffer",
+  Http2Session::LogIO(Session(), this, "Writing from Inline Buffer",
                       reinterpret_cast<char*>(mTxInlineFrame.get()),
                       transmittedCount);
 
@@ -942,16 +955,16 @@ nsresult Http2Stream::TransmitFrame(const char* buf, uint32_t* countUsed,
 
     // If there is already data buffered, just add to that to form
     // a single TLS Application Data Record - otherwise skip the memcpy
-    if (mSession->AmountOfOutputBuffered()) {
-      rv = mSession->BufferOutput(buf, mTxStreamFrameSize, &transmittedCount);
+    if (Session()->AmountOfOutputBuffered()) {
+      rv = Session()->BufferOutput(buf, mTxStreamFrameSize, &transmittedCount);
     } else {
-      rv = mSession->OnReadSegment(buf, mTxStreamFrameSize, &transmittedCount);
+      rv = Session()->OnReadSegment(buf, mTxStreamFrameSize, &transmittedCount);
     }
 
     LOG3(
         ("Http2Stream::TransmitFrame for regular session=%p "
          "stream=%p result %" PRIx32 " len=%d",
-         mSession, this, static_cast<uint32_t>(rv), transmittedCount));
+         Session(), this, static_cast<uint32_t>(rv), transmittedCount));
 
     MOZ_ASSERT(rv != NS_BASE_STREAM_WOULD_BLOCK,
                "inconsistent stream commitment result");
@@ -961,14 +974,14 @@ nsresult Http2Stream::TransmitFrame(const char* buf, uint32_t* countUsed,
     MOZ_ASSERT(transmittedCount == mTxStreamFrameSize,
                "inconsistent stream commitment count");
 
-    Http2Session::LogIO(mSession, this, "Writing from Transaction Buffer", buf,
+    Http2Session::LogIO(Session(), this, "Writing from Transaction Buffer", buf,
                         transmittedCount);
 
     *countUsed += mTxStreamFrameSize;
   }
 
   if (!mAttempting0RTT) {
-    mSession->FlushOutputQueue();
+    Session()->FlushOutputQueue();
   }
 
   // calling this will trigger waiting_for if mRequestBodyLenRemaining is 0
@@ -1000,9 +1013,9 @@ void Http2Stream::GenerateDataFrameHeader(uint32_t dataLength, bool lastFrame) {
     if (dataLength) SetSentFin(true);
   }
 
-  mSession->CreateFrameHeader(mTxInlineFrame.get(), dataLength,
-                              Http2Session::FRAME_TYPE_DATA, frameFlags,
-                              mStreamID);
+  Session()->CreateFrameHeader(mTxInlineFrame.get(), dataLength,
+                               Http2Session::FRAME_TYPE_DATA, frameFlags,
+                               mStreamID);
 
   mTxInlineFrameUsed = Http2Session::kFrameHeaderBytes;
   mTxStreamFrameSize = dataLength;
@@ -1048,11 +1061,16 @@ nsresult Http2Stream::ConvertResponseHeaders(Http2Decompressor* decompressor,
         httpResponseCode));
   if (mIsTunnel) {
     LOG3(("Http2Stream %p Tunnel Response code %d", this, httpResponseCode));
-    if ((httpResponseCode / 100) != 2) {
+    // 1xx response is simply skipeed and a final response is expected.
+    // 2xx response needs to be encrypted.
+    if ((httpResponseCode / 100) > 2) {
       MapStreamToPlainText();
     }
-    MapStreamToHttpConnection(aHeadersOut, httpResponseCode);
-    ClearTransactionsBlockedOnTunnel();
+    if (MapStreamToHttpConnection(aHeadersOut, httpResponseCode)) {
+      // Process transactions only if we have a final response, i.e., response
+      // code >= 200.
+      ClearTransactionsBlockedOnTunnel();
+    }
   } else if (mIsWebsocket) {
     LOG3(("Http2Stream %p websocket response code %d", this, httpResponseCode));
     if (httpResponseCode == 200) {
@@ -1060,16 +1078,9 @@ nsresult Http2Stream::ConvertResponseHeaders(Http2Decompressor* decompressor,
     }
   }
 
-  if (httpResponseCode == 101) {
-    // 8.1.1 of h2 disallows 101.. throw PROTOCOL_ERROR on stream
-    LOG3(("Http2Stream::ConvertResponseHeaders %p Error - status == 101\n",
-          this));
-    return NS_ERROR_ILLEGAL_VALUE;
-  }
-
   if (httpResponseCode == 421) {
     // Origin Frame requires 421 to remove this origin from the origin set
-    mSession->Received421(mTransaction->ConnectionInfo());
+    Session()->Received421(mTransaction->ConnectionInfo());
   }
 
   if (aHeadersIn.Length() && aHeadersOut.Length()) {
@@ -1164,6 +1175,7 @@ void Http2Stream::Close(nsresult reason) {
   ClearPushSource();
 
   mTransaction->Close(reason);
+  mSession = nullptr;
 }
 
 void Http2Stream::SetResponseIsComplete() {
@@ -1191,7 +1203,7 @@ void Http2Stream::SetAllHeadersReceived() {
 }
 
 bool Http2Stream::AllowFlowControlledWrite() {
-  return (mSession->ServerSessionWindow() > 0) && (mServerReceiveWindow > 0);
+  return (Session()->ServerSessionWindow() > 0) && (mServerReceiveWindow > 0);
 }
 
 void Http2Stream::UpdateServerReceiveWindow(int32_t delta) {
@@ -1202,7 +1214,7 @@ void Http2Stream::UpdateServerReceiveWindow(int32_t delta) {
         ("Http2Stream::UpdateServerReceived UnPause %p 0x%X "
          "Open stream window\n",
          this, mStreamID));
-    mSession->TransactionHasDataToWrite(this);
+    Session()->TransactionHasDataToWrite(this);
   }
 }
 
@@ -1259,7 +1271,7 @@ static uint32_t GetPriorityDependencyFromTransaction(nsHttpTransaction* trans) {
 }
 
 void Http2Stream::UpdatePriorityDependency() {
-  if (!mSession->UseH2Deps()) {
+  if (!Session()->UseH2Deps()) {
     return;
   }
 
@@ -1287,7 +1299,7 @@ void Http2Stream::UpdatePriorityDependency() {
   mPriorityDependency = GetPriorityDependencyFromTransaction(trans);
 
   if (gHttpHandler->ActiveTabPriority() &&
-      mTransactionTabId != mCurrentForegroundTabOuterContentWindowId &&
+      mTransactionTabId != mCurrentTopBrowsingContextId &&
       mPriorityDependency != Http2Session::kUrgentStartGroupID) {
     LOG3(
         ("Http2Stream::UpdatePriorityDependency %p "
@@ -1304,7 +1316,7 @@ void Http2Stream::UpdatePriorityDependency() {
        this, mPriorityDependency));
 }
 
-void Http2Stream::TopLevelOuterContentWindowIdChanged(uint64_t windowId) {
+void Http2Stream::TopBrowsingContextIdChanged(uint64_t id) {
   if (!mStreamID) {
     // For pushed streams, we ignore the direct call from the session and
     // instead let it come to the internal function from the pushed stream, so
@@ -1312,21 +1324,20 @@ void Http2Stream::TopLevelOuterContentWindowIdChanged(uint64_t windowId) {
     return;
   }
 
-  TopLevelOuterContentWindowIdChangedInternal(windowId);
+  TopBrowsingContextIdChangedInternal(id);
 }
 
-void Http2Stream::TopLevelOuterContentWindowIdChangedInternal(
-    uint64_t windowId) {
+void Http2Stream::TopBrowsingContextIdChangedInternal(uint64_t id) {
   MOZ_ASSERT(gHttpHandler->ActiveTabPriority());
 
   LOG3(
-      ("Http2Stream::TopLevelOuterContentWindowIdChanged "
-       "%p windowId=%" PRIx64 "\n",
-       this, windowId));
+      ("Http2Stream::TopBrowsingContextIdChangedInternal "
+       "%p bcId=%" PRIx64 "\n",
+       this, id));
 
-  mCurrentForegroundTabOuterContentWindowId = windowId;
+  mCurrentTopBrowsingContextId = id;
 
-  if (!mSession->UseH2Deps()) {
+  if (!Session()->UseH2Deps()) {
     return;
   }
 
@@ -1336,10 +1347,10 @@ void Http2Stream::TopLevelOuterContentWindowIdChangedInternal(
     return;
   }
 
-  if (mTransactionTabId != mCurrentForegroundTabOuterContentWindowId) {
+  if (mTransactionTabId != mCurrentTopBrowsingContextId) {
     mPriorityDependency = Http2Session::kBackgroundGroupID;
     LOG3(
-        ("Http2Stream::TopLevelOuterContentWindowIdChanged %p "
+        ("Http2Stream::TopBrowsingContextIdChangedInternal %p "
          "move into background group.\n",
          this));
 
@@ -1352,7 +1363,7 @@ void Http2Stream::TopLevelOuterContentWindowIdChangedInternal(
 
     mPriorityDependency = GetPriorityDependencyFromTransaction(trans);
     LOG3(
-        ("Http2Stream::TopLevelOuterContentWindowIdChanged %p "
+        ("Http2Stream::TopBrowsingContextIdChangedInternal %p "
          "depends on stream 0x%X\n",
          this, mPriorityDependency));
   }
@@ -1362,8 +1373,8 @@ void Http2Stream::TopLevelOuterContentWindowIdChangedInternal(
     modifyStreamID = mPushSource->StreamID();
   }
   if (modifyStreamID) {
-    mSession->SendPriorityFrame(modifyStreamID, mPriorityDependency,
-                                mPriorityWeight);
+    Session()->SendPriorityFrame(modifyStreamID, mPriorityDependency,
+                                 mPriorityWeight);
   }
 }
 
@@ -1432,7 +1443,7 @@ nsresult Http2Stream::OnReadSegment(const char* buf, uint32_t count,
       }
 
       if (mRequestHeadersDone && !mOpenGenerated) {
-        if (!mSession->TryToActivate(this)) {
+        if (!Session()->TryToActivate(this)) {
           LOG3(("Http2Stream::OnReadSegment %p cannot activate now. queued.\n",
                 this));
           return *countRead ? NS_OK : NS_BASE_STREAM_WOULD_BLOCK;
@@ -1467,7 +1478,7 @@ nsresult Http2Stream::OnReadSegment(const char* buf, uint32_t count,
             ("Http2Stream this=%p, id 0x%X request body suspended because "
              "remote window is stream=%" PRId64 " session=%" PRId64 ".\n",
              this, mStreamID, mServerReceiveWindow,
-             mSession->ServerSessionWindow()));
+             Session()->ServerSessionWindow()));
         mBlockedOnRwin = true;
         return NS_BASE_STREAM_WOULD_BLOCK;
       }
@@ -1478,14 +1489,17 @@ nsresult Http2Stream::OnReadSegment(const char* buf, uint32_t count,
       // Its amazing we send anything at all.
       dataLength = std::min(count, mChunkSize);
 
-      if (dataLength > Http2Session::kMaxFrameData)
+      if (dataLength > Http2Session::kMaxFrameData) {
         dataLength = Http2Session::kMaxFrameData;
+      }
 
-      if (dataLength > mSession->ServerSessionWindow())
-        dataLength = static_cast<uint32_t>(mSession->ServerSessionWindow());
+      if (dataLength > Session()->ServerSessionWindow()) {
+        dataLength = static_cast<uint32_t>(Session()->ServerSessionWindow());
+      }
 
-      if (dataLength > mServerReceiveWindow)
+      if (dataLength > mServerReceiveWindow) {
         dataLength = static_cast<uint32_t>(mServerReceiveWindow);
+      }
 
       LOG3(
           ("Http2Stream this=%p id 0x%X send calculation "
@@ -1493,10 +1507,10 @@ nsresult Http2Stream::OnReadSegment(const char* buf, uint32_t count,
            " session window=%" PRId64 " "
            "max frame=%d USING=%u\n",
            this, mStreamID, count, mChunkSize, mServerReceiveWindow,
-           mSession->ServerSessionWindow(), Http2Session::kMaxFrameData,
+           Session()->ServerSessionWindow(), Http2Session::kMaxFrameData,
            dataLength));
 
-      mSession->DecrementServerSessionWindow(dataLength);
+      Session()->DecrementServerSessionWindow(dataLength);
       mServerReceiveWindow -= dataLength;
 
       LOG3(("Http2Stream %p id 0x%x request len remaining %" PRId64 ", "
@@ -1567,7 +1581,7 @@ nsresult Http2Stream::OnWriteSegment(char* buf, uint32_t count,
     rv = mPushSource->GetBufferedData(buf, count, countWritten);
     if (NS_FAILED(rv)) return rv;
 
-    mSession->ConnectPushedStream(this);
+    Session()->ConnectPushedStream(this);
     return NS_OK;
   }
 
@@ -1625,13 +1639,13 @@ void Http2Stream::MapStreamToPlainText() {
   qiTrans->ForcePlainText();
 }
 
-void Http2Stream::MapStreamToHttpConnection(const nsACString& aFlat407Headers,
+bool Http2Stream::MapStreamToHttpConnection(const nsACString& aFlat407Headers,
                                             int32_t aHttpResponseCode) {
   RefPtr<SpdyConnectTransaction> qiTrans(
       mTransaction->QuerySpdyConnectTransaction());
   MOZ_ASSERT(qiTrans);
 
-  qiTrans->MapStreamToHttpConnection(
+  return qiTrans->MapStreamToHttpConnection(
       mSocketTransport, mTransaction->ConnectionInfo(), aFlat407Headers,
       mIsTunnel ? aHttpResponseCode : -1);
 }
