@@ -8,6 +8,7 @@
 
 #include "Http3Session.h"
 #include "SharedCertVerifier.h"
+#include "nsISocketProvider.h"
 #include "nsIWebProgressListener.h"
 #include "nsNSSComponent.h"
 #include "nsWeakReference.h"
@@ -65,14 +66,7 @@ void QuicSocketControl::HandshakeCompleted() {
 
   uint32_t state = nsIWebProgressListener::STATE_IS_SECURE;
 
-  bool distrustImminent;
   MutexAutoLock lock(mMutex);
-  nsresult rv =
-      IsCertificateDistrustImminent(mSucceededCertChain, distrustImminent);
-
-  if (NS_SUCCEEDED(rv) && distrustImminent) {
-    state |= nsIWebProgressListener::STATE_CERT_DISTRUST_IMMINENT;
-  }
 
   // If we're here, the TLS handshake has succeeded. Thus if any of these
   // booleans are true, the user has added an override for a certificate error.
@@ -85,6 +79,7 @@ void QuicSocketControl::HandshakeCompleted() {
 }
 
 void QuicSocketControl::SetNegotiatedNPN(const nsACString& aValue) {
+  MutexAutoLock lock(mMutex);
   mNegotiatedNPN = aValue;
   mNPNCompleted = true;
 }
@@ -95,6 +90,7 @@ void QuicSocketControl::SetInfo(uint16_t aCipherSuite,
   SSLCipherSuiteInfo cipherInfo;
   if (SSL_GetCipherSuiteInfo(aCipherSuite, &cipherInfo, sizeof cipherInfo) ==
       SECSuccess) {
+    MutexAutoLock lock(mMutex);
     mHaveCipherSuiteAndProtocol = true;
     mCipherSuite = aCipherSuite;
     mProtocolVersion = aProtocolVersion & 0xFF;
@@ -103,5 +99,33 @@ void QuicSocketControl::SetInfo(uint16_t aCipherSuite,
   }
 }
 
+NS_IMETHODIMP QuicSocketControl::GetPeerId(nsACString& aResult) {
+  if (!mPeerId.IsEmpty()) {
+    aResult.Assign(mPeerId);
+    return NS_OK;
+  }
+
+  if (mProviderFlags &
+      nsISocketProvider::ANONYMOUS_CONNECT) {  // See bug 466080
+    mPeerId.AppendLiteral("anon:");
+  }
+  if (mProviderFlags & nsISocketProvider::NO_PERMANENT_STORAGE) {
+    mPeerId.AppendLiteral("private:");
+  }
+  if (mProviderFlags & nsISocketProvider::BE_CONSERVATIVE) {
+    mPeerId.AppendLiteral("beConservative:");
+  }
+
+  mPeerId.Append(GetHostName());
+  mPeerId.Append(':');
+  mPeerId.AppendInt(GetPort());
+  nsAutoCString suffix;
+  GetOriginAttributes().CreateSuffix(suffix);
+  mPeerId.Append(suffix);
+
+  aResult.Assign(mPeerId);
+  return NS_OK;
+}
+
 }  // namespace net
 }  // namespace mozilla
