@@ -294,11 +294,44 @@ class MessageEvent : public WebSocketEvent {
   bool mBinary;
 };
 
-mozilla::ipc::IPCResult WebSocketChannelChild::RecvOnMessageAvailable(
-    const nsCString& aMsg) {
-  mEventQ->RunOrEnqueue(new EventTargetDispatcher(
-      this, new MessageEvent(aMsg, false), mTargetThread));
+bool WebSocketChannelChild::RecvOnMessageAvailableInternal(
+    const nsDependentCSubstring& aMsg, bool aMoreData, bool aBinary) {
+  if (aMoreData) {
+    return mReceivedMsgBuffer.Append(aMsg, fallible);
+  }
 
+  if (!mReceivedMsgBuffer.Append(aMsg, fallible)) {
+    return false;
+  }
+
+  mEventQ->RunOrEnqueue(new EventTargetDispatcher(
+      this, new MessageEvent(mReceivedMsgBuffer, aBinary), mTargetThread));
+  mReceivedMsgBuffer.Truncate();
+  return true;
+}
+
+class OnErrorEvent : public WebSocketEvent {
+ public:
+  OnErrorEvent() = default;
+
+  void Run(WebSocketChannelChild* aChild) override { aChild->OnError(); }
+};
+
+void WebSocketChannelChild::OnError() {
+  LOG(("WebSocketChannelChild::OnError() %p", this));
+  if (mListenerMT) {
+    AutoEventEnqueuer ensureSerialDispatch(mEventQ);
+    Unused << mListenerMT->mListener->OnError();
+  }
+}
+
+mozilla::ipc::IPCResult WebSocketChannelChild::RecvOnMessageAvailable(
+    const nsDependentCSubstring& aMsg, const bool& aMoreData) {
+  if (!RecvOnMessageAvailableInternal(aMsg, aMoreData, false)) {
+    LOG(("WebSocketChannelChild %p append message failed", this));
+    mEventQ->RunOrEnqueue(
+        new EventTargetDispatcher(this, new OnErrorEvent(), mTargetThread));
+  }
   return IPC_OK();
 }
 
@@ -319,10 +352,12 @@ void WebSocketChannelChild::OnMessageAvailable(const nsCString& aMsg) {
 }
 
 mozilla::ipc::IPCResult WebSocketChannelChild::RecvOnBinaryMessageAvailable(
-    const nsCString& aMsg) {
-  mEventQ->RunOrEnqueue(new EventTargetDispatcher(
-      this, new MessageEvent(aMsg, true), mTargetThread));
-
+    const nsDependentCSubstring& aMsg, const bool& aMoreData) {
+  if (!RecvOnMessageAvailableInternal(aMsg, aMoreData, true)) {
+    LOG(("WebSocketChannelChild %p append message failed", this));
+    mEventQ->RunOrEnqueue(
+        new EventTargetDispatcher(this, new OnErrorEvent(), mTargetThread));
+  }
   return IPC_OK();
 }
 
