@@ -147,13 +147,13 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
     opCreateHTMLElement opeation(
         content, aName, aAttributes, aCreator.html, aIntendedParent,
         (!!mSpeculativeLoadStage) ? mozilla::dom::FROM_PARSER_NETWORK
-                                  : dom::FROM_PARSER_DOCUMENT_WRITE);
+                                  : mozilla::dom::FROM_PARSER_DOCUMENT_WRITE);
     treeOp->Init(mozilla::AsVariant(opeation));
   } else if (aNamespace == kNameSpaceID_SVG) {
     opCreateSVGElement operation(
         content, aName, aAttributes, aCreator.svg, aIntendedParent,
         (!!mSpeculativeLoadStage) ? mozilla::dom::FROM_PARSER_NETWORK
-                                  : dom::FROM_PARSER_DOCUMENT_WRITE);
+                                  : mozilla::dom::FROM_PARSER_DOCUMENT_WRITE);
     treeOp->Init(mozilla::AsVariant(operation));
   } else {
     // kNameSpaceID_MathML
@@ -167,7 +167,7 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
 
   // Start wall of code for speculative loading and line numbers
 
-  if (mSpeculativeLoadStage) {
+  if (mSpeculativeLoadStage && mode != IN_TEMPLATE) {
     switch (aNamespace) {
       case kNameSpaceID_XHTML:
         if (nsGkAtoms::img == aName) {
@@ -186,7 +186,8 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
             nsHtml5String sizes =
                 aAttributes->getValue(nsHtml5AttributeName::ATTR_SIZES);
             mSpeculativeLoadQueue.AppendElement()->InitImage(
-                url, crossOrigin, referrerPolicy, srcset, sizes, false);
+                url, crossOrigin, /* aMedia = */ nullptr, referrerPolicy,
+                srcset, sizes, false);
           }
         } else if (nsGkAtoms::source == aName) {
           nsHtml5String srcset =
@@ -236,9 +237,9 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
             bool noModule =
                 aAttributes->contains(nsHtml5AttributeName::ATTR_NOMODULE);
             mSpeculativeLoadQueue.AppendElement()->InitScript(
-                url, charset, type, crossOrigin, integrity, referrerPolicy,
-                mode == nsHtml5TreeBuilder::IN_HEAD, async, defer, noModule,
-                false);
+                url, charset, type, crossOrigin, /* aMedia = */ nullptr,
+                integrity, referrerPolicy, mode == nsHtml5TreeBuilder::IN_HEAD,
+                async, defer, noModule, false);
             mCurrentHtmlScriptIsAsyncOrDefer = async || defer;
           }
         } else if (nsGkAtoms::link == aName) {
@@ -259,8 +260,10 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
                     aAttributes->getValue(nsHtml5AttributeName::ATTR_INTEGRITY);
                 nsHtml5String referrerPolicy = aAttributes->getValue(
                     nsHtml5AttributeName::ATTR_REFERRERPOLICY);
+                nsHtml5String media =
+                    aAttributes->getValue(nsHtml5AttributeName::ATTR_MEDIA);
                 mSpeculativeLoadQueue.AppendElement()->InitStyle(
-                    url, charset, crossOrigin, referrerPolicy, integrity,
+                    url, charset, crossOrigin, media, referrerPolicy, integrity,
                     false);
               }
             } else if (rel.LowerCaseEqualsASCII("preconnect")) {
@@ -287,6 +290,8 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
                     aAttributes->getValue(nsHtml5AttributeName::ATTR_INTEGRITY);
                 nsHtml5String referrerPolicy = aAttributes->getValue(
                     nsHtml5AttributeName::ATTR_REFERRERPOLICY);
+                nsHtml5String media =
+                    aAttributes->getValue(nsHtml5AttributeName::ATTR_MEDIA);
 
                 // Note that respective speculative loaders for scripts and
                 // styles check all additional attributes to be equal to use the
@@ -299,26 +304,27 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
                   nsHtml5String type =
                       aAttributes->getValue(nsHtml5AttributeName::ATTR_TYPE);
                   mSpeculativeLoadQueue.AppendElement()->InitScript(
-                      url, charset, type, crossOrigin, integrity,
+                      url, charset, type, crossOrigin, media, integrity,
                       referrerPolicy, mode == nsHtml5TreeBuilder::IN_HEAD,
                       false, false, false, true);
                 } else if (as.LowerCaseEqualsASCII("style")) {
                   mSpeculativeLoadQueue.AppendElement()->InitStyle(
-                      url, charset, crossOrigin, referrerPolicy, integrity,
-                      true);
+                      url, charset, crossOrigin, media, referrerPolicy,
+                      integrity, true);
                 } else if (as.LowerCaseEqualsASCII("image")) {
                   nsHtml5String srcset = aAttributes->getValue(
                       nsHtml5AttributeName::ATTR_IMAGESRCSET);
                   nsHtml5String sizes = aAttributes->getValue(
                       nsHtml5AttributeName::ATTR_IMAGESIZES);
                   mSpeculativeLoadQueue.AppendElement()->InitImage(
-                      url, crossOrigin, referrerPolicy, srcset, sizes, true);
+                      url, crossOrigin, media, referrerPolicy, srcset, sizes,
+                      true);
                 } else if (as.LowerCaseEqualsASCII("font")) {
                   mSpeculativeLoadQueue.AppendElement()->InitFont(
-                      url, crossOrigin, referrerPolicy);
+                      url, crossOrigin, media, referrerPolicy);
                 } else if (as.LowerCaseEqualsASCII("fetch")) {
                   mSpeculativeLoadQueue.AppendElement()->InitFetch(
-                      url, crossOrigin, referrerPolicy);
+                      url, crossOrigin, media, referrerPolicy);
                 }
                 // Other "as" values will be supported later.
               }
@@ -329,7 +335,7 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
               aAttributes->getValue(nsHtml5AttributeName::ATTR_POSTER);
           if (url) {
             mSpeculativeLoadQueue.AppendElement()->InitImage(
-                url, nullptr, nullptr, nullptr, nullptr, false);
+                url, nullptr, nullptr, nullptr, nullptr, nullptr, false);
           }
         } else if (nsGkAtoms::style == aName) {
           mImportScanner.Start();
@@ -379,10 +385,13 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
       case kNameSpaceID_SVG:
         if (nsGkAtoms::image == aName) {
           nsHtml5String url =
-              aAttributes->getValue(nsHtml5AttributeName::ATTR_XLINK_HREF);
+              aAttributes->getValue(nsHtml5AttributeName::ATTR_HREF);
+          if (!url) {
+            url = aAttributes->getValue(nsHtml5AttributeName::ATTR_XLINK_HREF);
+          }
           if (url) {
             mSpeculativeLoadQueue.AppendElement()->InitImage(
-                url, nullptr, nullptr, nullptr, nullptr, false);
+                url, nullptr, nullptr, nullptr, nullptr, nullptr, false);
           }
         } else if (nsGkAtoms::script == aName) {
           nsHtml5TreeOperation* treeOp =
@@ -397,7 +406,10 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
           treeOp->Init(mozilla::AsVariant(operation));
 
           nsHtml5String url =
-              aAttributes->getValue(nsHtml5AttributeName::ATTR_XLINK_HREF);
+              aAttributes->getValue(nsHtml5AttributeName::ATTR_HREF);
+          if (!url) {
+            url = aAttributes->getValue(nsHtml5AttributeName::ATTR_XLINK_HREF);
+          }
           if (url) {
             nsHtml5String type =
                 aAttributes->getValue(nsHtml5AttributeName::ATTR_TYPE);
@@ -408,9 +420,9 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
             nsHtml5String referrerPolicy = aAttributes->getValue(
                 nsHtml5AttributeName::ATTR_REFERRERPOLICY);
             mSpeculativeLoadQueue.AppendElement()->InitScript(
-                url, nullptr, type, crossOrigin, integrity, referrerPolicy,
-                mode == nsHtml5TreeBuilder::IN_HEAD, false, false, false,
-                false);
+                url, nullptr, type, crossOrigin, /* aMedia = */ nullptr,
+                integrity, referrerPolicy, mode == nsHtml5TreeBuilder::IN_HEAD,
+                false, false, false, false);
           }
         } else if (nsGkAtoms::style == aName) {
           mImportScanner.Start();
@@ -469,7 +481,7 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
           opProcessOfflineManifest operation(ToNewUnicode(urlString));
           treeOp->Init(mozilla::AsVariant(operation));
         } else {
-          opProcessOfflineManifest operation(ToNewUnicode(EmptyString()));
+          opProcessOfflineManifest operation(ToNewUnicode(u""_ns));
           treeOp->Init(mozilla::AsVariant(operation));
         }
       } else if (nsGkAtoms::base == aName && mViewSource) {
@@ -602,7 +614,7 @@ void nsHtml5TreeBuilder::appendElement(nsIContentHandle* aChild,
   if (mBuilder) {
     nsresult rv = nsHtml5TreeOperation::Append(
         static_cast<nsIContent*>(aChild), static_cast<nsIContent*>(aParent),
-        mBuilder);
+        mozilla::dom::FROM_PARSER_FRAGMENT, mBuilder);
     if (NS_FAILED(rv)) {
       MarkAsBrokenAndRequestSuspensionWithBuilder(rv);
     }
@@ -614,7 +626,11 @@ void nsHtml5TreeBuilder::appendElement(nsIContentHandle* aChild,
     MarkAsBrokenAndRequestSuspensionWithoutBuilder(NS_ERROR_OUT_OF_MEMORY);
     return;
   }
-  opAppend operation(aChild, aParent);
+
+  opAppend operation(aChild, aParent,
+                     (!!mSpeculativeLoadStage)
+                         ? mozilla::dom::FROM_PARSER_NETWORK
+                         : mozilla::dom::FROM_PARSER_DOCUMENT_WRITE);
   treeOp->Init(mozilla::AsVariant(operation));
 }
 
@@ -738,8 +754,7 @@ void nsHtml5TreeBuilder::appendCharacters(nsIContentHandle* aParent,
   memcpy(bufferCopy.get(), aBuffer, aLength * sizeof(char16_t));
 
   if (mImportScanner.ShouldScan()) {
-    nsTArray<nsString> imports =
-        mImportScanner.Scan(MakeSpan(aBuffer, aLength));
+    nsTArray<nsString> imports = mImportScanner.Scan(Span(aBuffer, aLength));
     for (nsString& url : imports) {
       mSpeculativeLoadQueue.AppendElement()->InitImportStyle(std::move(url));
     }
@@ -1090,17 +1105,7 @@ void nsHtml5TreeBuilder::elementPopped(int32_t aNamespace, nsAtom* aName,
     }
     return;
   }
-  // we now have only HTML
-  if (aName == nsGkAtoms::meta && !fragment && !mBuilder) {
-    nsHtml5TreeOperation* treeOp = mOpQueue.AppendElement(mozilla::fallible);
-    if (MOZ_UNLIKELY(!treeOp)) {
-      MarkAsBrokenAndRequestSuspensionWithoutBuilder(NS_ERROR_OUT_OF_MEMORY);
-      return;
-    }
-    opProcessMeta operation(aElement);
-    treeOp->Init(mozilla::AsVariant(operation));
-    return;
-  }
+
   if (mSpeculativeLoadStage && aName == nsGkAtoms::picture) {
     // mSpeculativeLoadStage is non-null only in the off-the-main-thread
     // tree builder, which handles the network stream
@@ -1576,9 +1581,9 @@ void nsHtml5TreeBuilder::errStartTagWithSelectOpen(nsAtom* aName) {
   }
 }
 
-void nsHtml5TreeBuilder::errBadStartTagInHead(nsAtom* aName) {
+void nsHtml5TreeBuilder::errBadStartTagInNoscriptInHead(nsAtom* aName) {
   if (MOZ_UNLIKELY(mViewSource)) {
-    mViewSource->AddErrorToCurrentRun("errBadStartTagInHead2", aName);
+    mViewSource->AddErrorToCurrentRun("errBadStartTagInNoscriptInHead", aName);
   }
 }
 
@@ -1596,7 +1601,7 @@ void nsHtml5TreeBuilder::errIsindex() {
 
 void nsHtml5TreeBuilder::errFooSeenWhenFooOpen(nsAtom* aName) {
   if (MOZ_UNLIKELY(mViewSource)) {
-    mViewSource->AddErrorToCurrentRun("errFooSeenWhenFooOpen", aName);
+    mViewSource->AddErrorToCurrentRun("errFooSeenWhenFooOpen2", aName);
   }
 }
 
@@ -1685,12 +1690,6 @@ void nsHtml5TreeBuilder::errHtmlStartTagInForeignContext(nsAtom* aName) {
   }
 }
 
-void nsHtml5TreeBuilder::errTableClosedWhileCaptionOpen() {
-  if (MOZ_UNLIKELY(mViewSource)) {
-    mViewSource->AddErrorToCurrentRun("errTableClosedWhileCaptionOpen");
-  }
-}
-
 void nsHtml5TreeBuilder::errNoTableRowToClose() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentRun("errNoTableRowToClose");
