@@ -23,7 +23,6 @@
 #include "nsIFragmentContentSink.h"
 #include "nsStreamUtils.h"
 #include "nsHTMLTokenizer.h"
-#include "nsDataHashtable.h"
 #include "nsXPCOMCIDInternal.h"
 #include "nsMimeTypes.h"
 #include "mozilla/CondVar.h"
@@ -260,7 +259,8 @@ nsParser::SetCommand(eParserCommands aParserCommand) {
  *  @param   aCharsetSource- the source of the charset
  */
 void nsParser::SetDocumentCharset(NotNull<const Encoding*> aCharset,
-                                  int32_t aCharsetSource) {
+                                  int32_t aCharsetSource,
+                                  bool aChannelHadCharset) {
   mCharset = aCharset;
   mCharsetSource = aCharsetSource;
   if (mParserContext && mParserContext->mScanner) {
@@ -795,7 +795,7 @@ nsresult nsParser::Parse(const nsAString& aSourceBuffer, void* aKey,
       // end fix for 40143
 
       pc->mContextType = CParserContext::eCTString;
-      pc->SetMimeType(NS_LITERAL_CSTRING("application/xml"));
+      pc->SetMimeType("application/xml"_ns);
       pc->mDTDMode = eDTDMode_full_standards;
 
       mUnusedInput.Truncate();
@@ -876,7 +876,7 @@ nsParser::ParseFragment(const nsAString& aSourceBuffer,
   } else {
     // Add an end tag chunk, so expat will read the whole source buffer,
     // and not worry about ']]' etc.
-    result = Parse(aSourceBuffer + NS_LITERAL_STRING("</"), &theContext, false);
+    result = Parse(aSourceBuffer + u"</"_ns, &theContext, false);
     fragSink->DidBuildContent();
 
     if (NS_SUCCEEDED(result)) {
@@ -1225,7 +1225,7 @@ static nsresult ParserWriteFunc(nsIInputStream* in, void* closure,
     // declaration to be entirely in the first network buffer. -- hsivonen
     const Encoding* encoding;
     size_t bomLength;
-    Tie(encoding, bomLength) = Encoding::ForBOM(MakeSpan(buf, count));
+    Tie(encoding, bomLength) = Encoding::ForBOM(Span(buf, count));
     Unused << bomLength;
     if (encoding) {
       // The decoder will swallow the BOM. The UTF-16 will re-sniff for
@@ -1245,7 +1245,7 @@ static nsresult ParserWriteFunc(nsIInputStream* in, void* closure,
       }
     }
 
-    pws->mParser->SetDocumentCharset(preferred, source);
+    pws->mParser->SetDocumentCharset(preferred, source, false);
     pws->mParser->SetSinkCharset(preferred);
   }
 
