@@ -11,7 +11,9 @@
 
 #include "mozilla/HashFunctions.h"
 #include "mozilla/Mutex.h"
+#include "mozilla/TaskQueue.h"
 #include "mozilla/TypedEnumBits.h"
+#include "nsIAsyncShutdown.h"
 #include "nsICertOverrideService.h"
 #include "nsIFile.h"
 #include "nsIObserver.h"
@@ -20,8 +22,11 @@
 #include "nsWeakReference.h"
 #include "secoidt.h"
 
-class nsCertOverride {
+class nsCertOverride final : public nsICertOverride {
  public:
+  NS_DECL_THREADSAFE_ISUPPORTS
+  NS_DECL_NSICERTOVERRIDE
+
   enum class OverrideBits {
     None = 0,
     Untrusted = nsICertOverrideService::ERROR_UNTRUSTED,
@@ -34,6 +39,7 @@ class nsCertOverride {
 
   nsCString mAsciiHost;
   int32_t mPort;
+  OriginAttributes mOriginAttributes;
   bool mIsTemporary;  // true: session only, false: stored on disk
   nsCString mFingerprint;
   OverrideBits mOverrideBits;
@@ -42,6 +48,9 @@ class nsCertOverride {
 
   static void convertBitsToString(OverrideBits ob, nsACString& str);
   static void convertStringToBits(const nsACString& str, OverrideBits& ob);
+
+ private:
+  ~nsCertOverride() = default;
 };
 
 MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS(nsCertOverride::OverrideBits)
@@ -59,16 +68,16 @@ class nsCertOverrideEntry final : public PLDHashEntryHdr {
   nsCertOverrideEntry(nsCertOverrideEntry&& toMove)
       : PLDHashEntryHdr(std::move(toMove)),
         mSettings(std::move(toMove.mSettings)),
-        mHostWithPort(std::move(toMove.mHostWithPort)) {}
+        mKeyString(std::move(toMove.mKeyString)) {}
 
   ~nsCertOverrideEntry() = default;
 
-  KeyType GetKey() const { return HostWithPortPtr(); }
+  KeyType GetKey() const { return KeyStringPtr(); }
 
-  KeyTypePointer GetKeyPointer() const { return HostWithPortPtr(); }
+  KeyTypePointer GetKeyPointer() const { return KeyStringPtr(); }
 
   bool KeyEquals(KeyTypePointer aKey) const {
-    return !strcmp(HostWithPortPtr(), aKey);
+    return !strcmp(KeyStringPtr(), aKey);
   }
 
   static KeyTypePointer KeyToPointer(KeyType aKey) { return aKey; }
@@ -80,47 +89,51 @@ class nsCertOverrideEntry final : public PLDHashEntryHdr {
   enum { ALLOW_MEMMOVE = false };
 
   // get methods
-  inline const nsCString& HostWithPort() const { return mHostWithPort; }
+  inline const nsCString& KeyString() const { return mKeyString; }
 
-  inline KeyTypePointer HostWithPortPtr() const { return mHostWithPort.get(); }
+  inline KeyTypePointer KeyStringPtr() const { return mKeyString.get(); }
 
-  nsCertOverride mSettings;
-  nsCString mHostWithPort;
+  RefPtr<nsCertOverride> mSettings;
+  nsCString mKeyString;
 };
 
 class nsCertOverrideService final : public nsICertOverrideService,
                                     public nsIObserver,
-                                    public nsSupportsWeakReference {
+                                    public nsSupportsWeakReference,
+                                    public nsIAsyncShutdownBlocker {
  public:
   NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSICERTOVERRIDESERVICE
   NS_DECL_NSIOBSERVER
+  NS_DECL_NSIASYNCSHUTDOWNBLOCKER
 
   nsCertOverrideService();
 
   nsresult Init();
   void RemoveAllTemporaryOverrides();
 
-  typedef void (*CertOverrideEnumerator)(const nsCertOverride& aSettings,
-                                         void* aUserData);
-
-  // aCert == null: return all overrides
-  // aCert != null: return overrides that match the given cert
-  nsresult EnumerateCertOverrides(nsIX509Cert* aCert,
-                                  CertOverrideEnumerator enumerator,
-                                  void* aUserData);
-
-  // Concates host name and the port number. If the port number is -1 then
+  // Concatenates host name and the port number. If the port number is -1 then
   // port 443 is automatically used. This method ensures there is always a port
   // number separated with colon.
   static void GetHostWithPort(const nsACString& aHostName, int32_t aPort,
-                              nsACString& _retval);
+                              nsACString& aRetval);
 
- protected:
+  // Concatenates host name, port number, and origin attributes.
+  static void GetKeyString(const nsACString& aHostName, int32_t aPort,
+                           const OriginAttributes& aOriginAttributes,
+                           nsACString& aRetval);
+
+  void AssertOnTaskQueue() const {
+    MOZ_ASSERT(mWriterTaskQueue->IsOnCurrentThread());
+  }
+
+  void RemoveShutdownBlocker();
+
+ private:
   ~nsCertOverrideService();
 
-  bool mDisableAllSecurityCheck;
   mozilla::Mutex mMutex;
+  bool mDisableAllSecurityCheck;
   nsCOMPtr<nsIFile> mSettingsFile;
   nsTHashtable<nsCertOverrideEntry> mSettingsTable;
 
@@ -131,11 +144,17 @@ class nsCertOverrideService final : public nsICertOverrideService,
   nsresult Read(const mozilla::MutexAutoLock& aProofOfLock);
   nsresult Write(const mozilla::MutexAutoLock& aProofOfLock);
   nsresult AddEntryToList(const nsACString& host, int32_t port,
+                          const OriginAttributes& aOriginAttributes,
                           nsIX509Cert* aCert, const bool aIsTemporary,
                           const nsACString& fingerprint,
                           nsCertOverride::OverrideBits ob,
                           const nsACString& dbKey,
                           const mozilla::MutexAutoLock& aProofOfLock);
+
+  RefPtr<mozilla::TaskQueue> mWriterTaskQueue;
+
+  // Only accessed on the main thread
+  uint64_t mPendingWriteCount;
 };
 
 #define NS_CERTOVERRIDE_CID                          \
