@@ -12,7 +12,6 @@
 #include "ScopedNSSTypes.h"
 #include "certt.h"
 #include "nsCOMPtr.h"
-#include "nsIASN1Object.h"
 #include "nsIClassInfo.h"
 #include "nsISerializable.h"
 #include "nsIX509Cert.h"
@@ -26,7 +25,6 @@ class DERArray;
 }  // namespace mozilla
 
 class nsINSSComponent;
-class nsIASN1Sequence;
 
 class nsNSSCertificate final : public nsIX509Cert,
                                public nsISerializable,
@@ -42,47 +40,30 @@ class nsNSSCertificate final : public nsIX509Cert,
   static nsNSSCertificate* Create(CERTCertificate* cert = nullptr);
   static nsNSSCertificate* ConstructFromDER(char* certDER, int derLen);
 
-  // This is a separate static method so nsNSSComponent can use it during NSS
-  // initialization. Other code should probably not use it.
-  static nsresult GetDbKey(const mozilla::UniqueCERTCertificate& cert,
-                           nsACString& aDbKey);
-
-  // Split a certificate chain into the root, intermediates (if any), and end
-  // entity. This method does so blindly, assuming that the current list object
-  // is ordered [end entity, intermediates..., root]. If that isn't true, this
-  // method will return the certificates at the two ends without regard to the
-  // actual chain of trust. Callers are encouraged to check, if there's any
-  // doubt.
+  // This method assumes that the current list object
+  // is ordered [end entity, intermediates..., root].
   // Will return error if used on self-signed or empty chains.
-  // This method requires that all arguments be empty, notably the list
-  // `aIntermediates` must be empty.
-  static nsresult SegmentCertificateChain(
+  // This method requires that the list `aIntermediates` must be empty.
+  static nsresult GetIntermediatesAsDER(
       /* int */ const nsTArray<RefPtr<nsIX509Cert>>& aCertList,
-      /* out */ nsCOMPtr<nsIX509Cert>& aRoot,
-      /* out */ nsTArray<RefPtr<nsIX509Cert>>& aIntermediates,
-      /* out */ nsCOMPtr<nsIX509Cert>& aEndEntity);
+      /* out */ nsTArray<nsTArray<uint8_t>>& aIntermediates);
 
-  // Obtain the root certificate of a certificate chain. This method does so
-  // blindly, as SegmentCertificateChain; the same restrictions apply. On an
+  // Obtain the root certificate of a certificate chain. On an
   // empty list, leaves aRoot empty and returns a failure.
+  // Assumes list is ordered [end entity, intermediates..., root].
   static nsresult GetRootCertificate(
       const nsTArray<RefPtr<nsIX509Cert>>& aCertList,
       /* out */ nsCOMPtr<nsIX509Cert>& aRoot);
 
  private:
-  virtual ~nsNSSCertificate();
+  virtual ~nsNSSCertificate() = default;
 
   mozilla::UniqueCERTCertificate mCert;
-  bool mPermDelete;
   uint32_t mCertType;
-  std::vector<nsString> mSubjectAltNames;
-  nsresult CreateASN1Struct(nsIASN1Object** aRetVal);
-  nsresult CreateTBSCertificateASN1Struct(nsIASN1Sequence** retSequence);
   nsresult GetSortableDate(PRTime aTime, nsAString& _aSortableDate);
   bool InitFromDER(char* certDER, int derLen);  // return false on failure
 
   nsresult GetCertificateHash(nsAString& aFingerprint, SECOidTag aHashAlg);
-  void GetSubjectAltNames();
 };
 
 namespace mozilla {
