@@ -10,6 +10,7 @@ use std::collections::{BTreeMap, BTreeSet};
 use crate::backend_macos as backend;
 #[cfg(target_os = "windows")]
 use crate::backend_windows as backend;
+use crate::util::*;
 use backend::*;
 
 use std::sync::mpsc::{channel, Receiver, Sender};
@@ -17,6 +18,15 @@ use std::thread;
 use std::thread::JoinHandle;
 use std::time::{Duration, Instant};
 
+/// Helper enum to differentiate between sessions on the modern slot and sessions on the legacy
+/// slot. The former is for EC keys and RSA keys that can be used with RSA-PSS whereas the latter is
+/// for RSA keys that cannot be used with RSA-PSS.
+#[derive(Clone, Copy, PartialEq)]
+pub enum SlotType {
+    Modern,
+    Legacy,
+}
+
 /// Helper type for sending `ManagerArguments` to the real `Manager`.
 type ManagerArgumentsSender = Sender<ManagerArguments>;
 /// Helper type for receiving `ManagerReturnValue`s from the real `Manager`.
@@ -26,9 +36,9 @@ type ManagerReturnValueReceiver = Receiver<ManagerReturnValue>;
 /// `ManagerArguments::Stop` is a special variant that stops the background thread and drops the
 /// `Manager`.
 enum ManagerArguments {
-    OpenSession,
+    OpenSession(SlotType),
     CloseSession(CK_SESSION_HANDLE),
-    CloseAllSessions,
+    CloseAllSessions(SlotType),
     StartSearch(CK_SESSION_HANDLE, Vec<(CK_ATTRIBUTE_TYPE, Vec<u8>)>),
     Search(CK_SESSION_HANDLE, usize),
     ClearSearch(CK_SESSION_HANDLE),
@@ -88,81 +98,95 @@ pub struct ManagerProxy {
 }
 
 impl ManagerProxy {
-    pub fn new() -> ManagerProxy {
+    pub fn new() -> Result<ManagerProxy, ()> {
         let (proxy_sender, manager_receiver) = channel();
         let (manager_sender, proxy_receiver) = channel();
-        let thread_handle = thread::spawn(move || {
-            let mut real_manager = Manager::new();
-            loop {
-                let arguments = match manager_receiver.recv() {
-                    Ok(arguments) => arguments,
-                    Err(e) => {
-                        error!("error recv()ing arguments from ManagerProxy: {}", e);
-                        break;
+        let thread_handle = thread::Builder::new()
+            .name("osclientcert".into())
+            .spawn(move || {
+                let mut real_manager = Manager::new();
+                loop {
+                    let arguments = match manager_receiver.recv() {
+                        Ok(arguments) => arguments,
+                        Err(e) => {
+                            error!("error recv()ing arguments from ManagerProxy: {}", e);
+                            break;
+                        }
+                    };
+                    let results = match arguments {
+                        ManagerArguments::OpenSession(slot_type) => {
+                            ManagerReturnValue::OpenSession(real_manager.open_session(slot_type))
+                        }
+                        ManagerArguments::CloseSession(session_handle) => {
+                            ManagerReturnValue::CloseSession(
+                                real_manager.close_session(session_handle),
+                            )
+                        }
+                        ManagerArguments::CloseAllSessions(slot_type) => {
+                            ManagerReturnValue::CloseAllSessions(
+                                real_manager.close_all_sessions(slot_type),
+                            )
+                        }
+                        ManagerArguments::StartSearch(session, attrs) => {
+                            ManagerReturnValue::StartSearch(
+                                real_manager.start_search(session, &attrs),
+                            )
+                        }
+                        ManagerArguments::Search(session, max_objects) => {
+                            ManagerReturnValue::Search(real_manager.search(session, max_objects))
+                        }
+                        ManagerArguments::ClearSearch(session) => {
+                            ManagerReturnValue::ClearSearch(real_manager.clear_search(session))
+                        }
+                        ManagerArguments::GetAttributes(object_handle, attr_types) => {
+                            ManagerReturnValue::GetAttributes(
+                                real_manager.get_attributes(object_handle, attr_types),
+                            )
+                        }
+                        ManagerArguments::StartSign(session, key_handle, params) => {
+                            ManagerReturnValue::StartSign(
+                                real_manager.start_sign(session, key_handle, params),
+                            )
+                        }
+                        ManagerArguments::GetSignatureLength(session, data) => {
+                            ManagerReturnValue::GetSignatureLength(
+                                real_manager.get_signature_length(session, &data),
+                            )
+                        }
+                        ManagerArguments::Sign(session, data) => {
+                            ManagerReturnValue::Sign(real_manager.sign(session, &data))
+                        }
+                        ManagerArguments::Stop => {
+                            debug!("ManagerArguments::Stop received - stopping Manager thread.");
+                            ManagerReturnValue::Stop(Ok(()))
+                        }
+                    };
+                    let stop_after_send = match &results {
+                        &ManagerReturnValue::Stop(_) => true,
+                        _ => false,
+                    };
+                    match manager_sender.send(results) {
+                        Ok(()) => {}
+                        Err(e) => {
+                            error!("error send()ing results from Manager: {}", e);
+                            break;
+                        }
                     }
-                };
-                let results = match arguments {
-                    ManagerArguments::OpenSession => {
-                        ManagerReturnValue::OpenSession(real_manager.open_session())
-                    }
-                    ManagerArguments::CloseSession(session_handle) => {
-                        ManagerReturnValue::CloseSession(real_manager.close_session(session_handle))
-                    }
-                    ManagerArguments::CloseAllSessions => {
-                        ManagerReturnValue::CloseAllSessions(real_manager.close_all_sessions())
-                    }
-                    ManagerArguments::StartSearch(session, attrs) => {
-                        ManagerReturnValue::StartSearch(real_manager.start_search(session, &attrs))
-                    }
-                    ManagerArguments::Search(session, max_objects) => {
-                        ManagerReturnValue::Search(real_manager.search(session, max_objects))
-                    }
-                    ManagerArguments::ClearSearch(session) => {
-                        ManagerReturnValue::ClearSearch(real_manager.clear_search(session))
-                    }
-                    ManagerArguments::GetAttributes(object_handle, attr_types) => {
-                        ManagerReturnValue::GetAttributes(
-                            real_manager.get_attributes(object_handle, attr_types),
-                        )
-                    }
-                    ManagerArguments::StartSign(session, key_handle, params) => {
-                        ManagerReturnValue::StartSign(
-                            real_manager.start_sign(session, key_handle, params),
-                        )
-                    }
-                    ManagerArguments::GetSignatureLength(session, data) => {
-                        ManagerReturnValue::GetSignatureLength(
-                            real_manager.get_signature_length(session, &data),
-                        )
-                    }
-                    ManagerArguments::Sign(session, data) => {
-                        ManagerReturnValue::Sign(real_manager.sign(session, &data))
-                    }
-                    ManagerArguments::Stop => {
-                        debug!("ManagerArguments::Stop received - stopping Manager thread.");
-                        ManagerReturnValue::Stop(Ok(()))
-                    }
-                };
-                let stop_after_send = match &results {
-                    &ManagerReturnValue::Stop(_) => true,
-                    _ => false,
-                };
-                match manager_sender.send(results) {
-                    Ok(()) => {}
-                    Err(e) => {
-                        error!("error send()ing results from Manager: {}", e);
+                    if stop_after_send {
                         break;
                     }
                 }
-                if stop_after_send {
-                    break;
-                }
+            });
+        match thread_handle {
+            Ok(thread_handle) => Ok(ManagerProxy {
+                sender: proxy_sender,
+                receiver: proxy_receiver,
+                thread_handle: Some(thread_handle),
+            }),
+            Err(e) => {
+                error!("error creating manager proxy: {}", e);
+                Err(())
             }
-        });
-        ManagerProxy {
-            sender: proxy_sender,
-            receiver: proxy_receiver,
-            thread_handle: Some(thread_handle),
         }
     }
 
@@ -184,10 +208,10 @@ impl ManagerProxy {
         Ok(result)
     }
 
-    pub fn open_session(&mut self) -> Result<CK_SESSION_HANDLE, ()> {
+    pub fn open_session(&mut self, slot_type: SlotType) -> Result<CK_SESSION_HANDLE, ()> {
         manager_proxy_fn_impl!(
             self,
-            ManagerArguments::OpenSession,
+            ManagerArguments::OpenSession(slot_type),
             ManagerReturnValue::OpenSession
         )
     }
@@ -200,10 +224,10 @@ impl ManagerProxy {
         )
     }
 
-    pub fn close_all_sessions(&mut self) -> Result<(), ()> {
+    pub fn close_all_sessions(&mut self, slot_type: SlotType) -> Result<(), ()> {
         manager_proxy_fn_impl!(
             self,
-            ManagerArguments::CloseAllSessions,
+            ManagerArguments::CloseAllSessions(slot_type),
             ManagerReturnValue::CloseAllSessions
         )
     }
@@ -305,12 +329,44 @@ impl ManagerProxy {
     }
 }
 
+// Determines if the attributes of a given search correspond to NSS looking for all certificates or
+// private keys. Returns true if so, and false otherwise.
+// These searches are of the form:
+//   { { type: CKA_TOKEN, value: [1] },
+//     { type: CKA_CLASS, value: [CKO_CERTIFICATE or CKO_PRIVATE_KEY, as serialized bytes] } }
+// (although not necessarily in that order - see nssToken_TraverseCertificates and
+// nssToken_FindPrivateKeys)
+fn search_is_for_all_certificates_or_keys(
+    attrs: &[(CK_ATTRIBUTE_TYPE, Vec<u8>)],
+) -> Result<bool, ()> {
+    if attrs.len() != 2 {
+        return Ok(false);
+    }
+    let token_bytes = vec![1 as u8];
+    let mut found_token = false;
+    let cko_certificate_bytes = serialize_uint(CKO_CERTIFICATE)?;
+    let cko_private_key_bytes = serialize_uint(CKO_PRIVATE_KEY)?;
+    let mut found_certificate_or_private_key = false;
+    for (attr_type, attr_value) in attrs.iter() {
+        if attr_type == &CKA_TOKEN && attr_value == &token_bytes {
+            found_token = true;
+        }
+        if attr_type == &CKA_CLASS
+            && (attr_value == &cko_certificate_bytes || attr_value == &cko_private_key_bytes)
+        {
+            found_certificate_or_private_key = true;
+        }
+    }
+    Ok(found_token && found_certificate_or_private_key)
+}
+
 /// The `Manager` keeps track of the state of this module with respect to the PKCS #11
 /// specification. This includes what sessions are open, which search and sign operations are
 /// ongoing, and what objects are known and by what handle.
 struct Manager {
-    /// A set of sessions. Sessions can be created (opened) and later closed.
-    sessions: BTreeSet<CK_SESSION_HANDLE>,
+    /// A map of session to session type (modern or legacy). Sessions can be created (opened) and
+    /// later closed.
+    sessions: BTreeMap<CK_SESSION_HANDLE, SlotType>,
     /// A map of searches to PKCS #11 object handles that match those searches.
     searches: BTreeMap<CK_SESSION_HANDLE, Vec<CK_OBJECT_HANDLE>>,
     /// A map of sign operations to a pair of the object handle and optionally some params being
@@ -334,8 +390,8 @@ struct Manager {
 
 impl Manager {
     pub fn new() -> Manager {
-        let mut manager = Manager {
-            sessions: BTreeSet::new(),
+        Manager {
+            sessions: BTreeMap::new(),
             searches: BTreeMap::new(),
             signs: BTreeMap::new(),
             objects: BTreeMap::new(),
@@ -344,15 +400,12 @@ impl Manager {
             next_session: 1,
             next_handle: 1,
             last_scan_time: None,
-        };
-        manager.maybe_find_new_objects();
-        manager
+        }
     }
 
-    /// When a new `Manager` is created and when a new session is opened (provided at least 3
-    /// seconds have elapsed since the last session was opened), this searches for certificates and
-    /// keys to expose. We de-duplicate previously-found certificates and keys by / keeping track of
-    /// their IDs.
+    /// When a new search session is opened (provided at least 3 seconds have elapsed since the
+    /// last session was opened), this searches for certificates and keys to expose. We
+    /// de-duplicate previously-found certificates and keys by keeping track of their IDs.
     fn maybe_find_new_objects(&mut self) {
         let now = Instant::now();
         match self.last_scan_time {
@@ -388,24 +441,29 @@ impl Manager {
         }
     }
 
-    pub fn open_session(&mut self) -> Result<CK_SESSION_HANDLE, ()> {
-        self.maybe_find_new_objects();
+    pub fn open_session(&mut self, slot_type: SlotType) -> Result<CK_SESSION_HANDLE, ()> {
         let next_session = self.next_session;
         self.next_session += 1;
-        self.sessions.insert(next_session);
+        self.sessions.insert(next_session, slot_type);
         Ok(next_session)
     }
 
     pub fn close_session(&mut self, session: CK_SESSION_HANDLE) -> Result<(), ()> {
-        if self.sessions.remove(&session) {
-            Ok(())
-        } else {
-            Err(())
-        }
+        self.sessions.remove(&session).ok_or(()).map(|_| ())
     }
 
-    pub fn close_all_sessions(&mut self) -> Result<(), ()> {
-        self.sessions.clear();
+    pub fn close_all_sessions(&mut self, slot_type: SlotType) -> Result<(), ()> {
+        let mut to_remove = Vec::new();
+        for (session, open_slot_type) in self.sessions.iter() {
+            if slot_type == *open_slot_type {
+                to_remove.push(*session);
+            }
+        }
+        for session in to_remove {
+            if self.sessions.remove(&session).is_none() {
+                return Err(());
+            }
+        }
         Ok(())
     }
 
@@ -424,9 +482,10 @@ impl Manager {
         session: CK_SESSION_HANDLE,
         attrs: &[(CK_ATTRIBUTE_TYPE, Vec<u8>)],
     ) -> Result<(), ()> {
-        if self.searches.contains_key(&session) {
-            return Err(());
-        }
+        let slot_type = match self.sessions.get(&session) {
+            Some(slot_type) => *slot_type,
+            None => return Err(()),
+        };
         // If the search is for an attribute we don't support, no objects will match. This check
         // saves us having to look through all of our objects.
         for (attr, _) in attrs {
@@ -435,9 +494,17 @@ impl Manager {
                 return Ok(());
             }
         }
+        // When NSS wants to find all certificates or all private keys, it will perform a search
+        // with a particular set of attributes. This implementation uses these searches as an
+        // indication for the backend to re-scan for new objects from tokens that may have been
+        // inserted or certificates that may have been imported into the OS. Since these searches
+        // are relatively rare, this minimizes the impact of doing these re-scans.
+        if search_is_for_all_certificates_or_keys(attrs)? {
+            self.maybe_find_new_objects();
+        }
         let mut handles = Vec::new();
         for (handle, object) in &self.objects {
-            if object.matches(attrs) {
+            if object.matches(slot_type, attrs) {
                 handles.push(*handle);
             }
         }
@@ -525,7 +592,7 @@ impl Manager {
     }
 
     pub fn get_signature_length(
-        &self,
+        &mut self,
         session: CK_SESSION_HANDLE,
         data: &[u8],
     ) -> Result<usize, ()> {
@@ -533,7 +600,7 @@ impl Manager {
             Some((key_handle, params)) => (key_handle, params),
             None => return Err(()),
         };
-        let key = match self.objects.get(&key_handle) {
+        let key = match self.objects.get_mut(&key_handle) {
             Some(Object::Key(key)) => key,
             _ => return Err(()),
         };
@@ -547,7 +614,7 @@ impl Manager {
             Some((key_handle, params)) => (key_handle, params),
             None => return Err(()),
         };
-        let key = match self.objects.get(&key_handle) {
+        let key = match self.objects.get_mut(&key_handle) {
             Some(Object::Key(key)) => key,
             _ => return Err(()),
         };
