@@ -30,7 +30,7 @@ function run_test() {
 
 add_task(async function test_verifyLogin() {
   // This test expects a clean slate -- no saved passphrase.
-  Services.logins.removeAllLogins();
+  Services.logins.removeAllUserFacingLogins();
   let johnHelper = track_collections_helper();
   let johnU = johnHelper.with_updated_collection;
 
